@@ -24,22 +24,29 @@ import (
 )
 
 type responseCallback = func(interface{})
+type rebuildCallback = func(uint32) []byte
+type serverStopCallback = func()
 
 type serviceType struct {
 	mutex           sync.Mutex
 	callbacks       map[uint32]responseCallback
+	rebuilds        map[int]rebuildCallback
+	serveStops      map[int]serverStopCallback
 	nextID          uint32
+	nextRebuildID   int
 	outgoingPackets chan outgoingPacket
 }
 
 type outgoingPacket struct {
-	bytes   []byte
-	isFinal bool
+	bytes    []byte
+	refCount int
 }
 
 func runService() {
 	service := serviceType{
 		callbacks:       make(map[uint32]responseCallback),
+		rebuilds:        make(map[int]rebuildCallback),
+		serveStops:      make(map[int]serverStopCallback),
 		outgoingPackets: make(chan outgoingPacket),
 	}
 	buffer := make([]byte, 16*1024)
@@ -56,8 +63,8 @@ func runService() {
 			os.Stdout.Write(packet.bytes)
 
 			// Only signal that this request is done when it has actually been written
-			if packet.isFinal {
-				waitGroup.Done()
+			if packet.refCount != 0 {
+				waitGroup.Add(packet.refCount)
 			}
 		}
 	}()
@@ -89,10 +96,12 @@ func runService() {
 			clone := append([]byte{}, packet...)
 			waitGroup.Add(1)
 			go func() {
-				if result := service.handleIncomingPacket(clone); result != nil {
-					service.outgoingPackets <- outgoingPacket{bytes: result, isFinal: true}
-				} else {
-					waitGroup.Done()
+				out := service.handleIncomingPacket(clone)
+				out.refCount--
+				if out.bytes != nil {
+					service.outgoingPackets <- out
+				} else if out.refCount != 0 {
+					waitGroup.Add(out.refCount)
 				}
 			}()
 		}
@@ -130,22 +139,24 @@ func (service *serviceType) sendRequest(request interface{}) interface{} {
 	return <-result
 }
 
-func (service *serviceType) handleIncomingPacket(bytes []byte) (result []byte) {
+func (service *serviceType) handleIncomingPacket(bytes []byte) (result outgoingPacket) {
 	p, ok := decodePacket(bytes)
 	if !ok {
-		return nil
+		return outgoingPacket{}
 	}
 
 	if p.isRequest {
 		// Catch panics in the code below so they get passed to the caller
 		defer func() {
 			if r := recover(); r != nil {
-				result = encodePacket(packet{
-					id: p.id,
-					value: map[string]interface{}{
-						"error": fmt.Sprintf("Panic: %v\n\n%s", r, debug.Stack()),
-					},
-				})
+				result = outgoingPacket{
+					bytes: encodePacket(packet{
+						id: p.id,
+						value: map[string]interface{}{
+							"error": fmt.Sprintf("Panic: %v\n\n%s", r, debug.Stack()),
+						},
+					}),
+				}
 			}
 		}()
 
@@ -157,7 +168,79 @@ func (service *serviceType) handleIncomingPacket(bytes []byte) (result []byte) {
 			return service.handleBuildRequest(p.id, request)
 
 		case "transform":
-			return service.handleTransformRequest(p.id, request)
+			return outgoingPacket{
+				bytes: service.handleTransformRequest(p.id, request),
+			}
+
+		case "rebuild":
+			rebuildID := request["rebuildID"].(int)
+			rebuild, ok := func() (rebuildCallback, bool) {
+				service.mutex.Lock()
+				defer service.mutex.Unlock()
+				rebuild, ok := service.rebuilds[rebuildID]
+				return rebuild, ok
+			}()
+			if !ok {
+				return outgoingPacket{
+					bytes: encodePacket(packet{
+						id: p.id,
+						value: map[string]interface{}{
+							"error": "Cannot rebuild",
+						},
+					}),
+				}
+			}
+			return outgoingPacket{
+				bytes: rebuild(p.id),
+			}
+
+		case "serve-stop":
+			serveID := request["serveID"].(int)
+			refCount := 0
+			serveStop := func() serverStopCallback {
+				// Only mutate the map while inside a mutex
+				service.mutex.Lock()
+				defer service.mutex.Unlock()
+				if serveStop, ok := service.serveStops[serveID]; ok {
+					// This serve is now considered finished. This matches the +1 reference
+					// count at the return of the serve call.
+					refCount = -1
+					return serveStop
+				}
+				return nil
+			}()
+			if serveStop != nil {
+				serveStop()
+			}
+			return outgoingPacket{
+				bytes: encodePacket(packet{
+					id:    p.id,
+					value: make(map[string]interface{}),
+				}),
+				refCount: refCount,
+			}
+
+		case "rebuild-dispose":
+			rebuildID := request["rebuildID"].(int)
+			refCount := 0
+			func() {
+				// Only mutate the map while inside a mutex
+				service.mutex.Lock()
+				defer service.mutex.Unlock()
+				if _, ok := service.rebuilds[rebuildID]; !ok {
+					// This build is now considered finished. This matches the +1 reference
+					// count at the return of the first build call for this rebuild chain.
+					refCount = -1
+					delete(service.rebuilds, rebuildID)
+				}
+			}()
+			return outgoingPacket{
+				bytes: encodePacket(packet{
+					id:    p.id,
+					value: make(map[string]interface{}),
+				}),
+				refCount: refCount,
+			}
 
 		case "error":
 			// This just exists so that errors during JavaScript API setup get printed
@@ -166,18 +249,22 @@ func (service *serviceType) handleIncomingPacket(bytes []byte) (result []byte) {
 			flags := decodeStringArray(request["flags"].([]interface{}))
 			msg := decodeMessageToPrivate(request["error"].(map[string]interface{}))
 			logger.PrintMessageToStderr(flags, msg)
-			return encodePacket(packet{
-				id:    p.id,
-				value: make(map[string]interface{}),
-			})
+			return outgoingPacket{
+				bytes: encodePacket(packet{
+					id:    p.id,
+					value: make(map[string]interface{}),
+				}),
+			}
 
 		default:
-			return encodePacket(packet{
-				id: p.id,
-				value: map[string]interface{}{
-					"error": fmt.Sprintf("Invalid command: %s", command),
-				},
-			})
+			return outgoingPacket{
+				bytes: encodePacket(packet{
+					id: p.id,
+					value: map[string]interface{}{
+						"error": fmt.Sprintf("Invalid command: %s", command),
+					},
+				}),
+			}
 		}
 	}
 
@@ -190,7 +277,7 @@ func (service *serviceType) handleIncomingPacket(bytes []byte) (result []byte) {
 	}()
 
 	callback(p.value)
-	return nil
+	return outgoingPacket{}
 }
 
 func encodeErrorPacket(id uint32, err error) []byte {
@@ -202,9 +289,11 @@ func encodeErrorPacket(id uint32, err error) []byte {
 	})
 }
 
-func (service *serviceType) handleBuildRequest(id uint32, request map[string]interface{}) []byte {
+func (service *serviceType) handleBuildRequest(id uint32, request map[string]interface{}) outgoingPacket {
 	key := request["key"].(int)
 	write := request["write"].(bool)
+	incremental := request["incremental"].(bool)
+	serve, isServe := request["serve"].(interface{})
 	flags := decodeStringArray(request["flags"].([]interface{}))
 
 	options, err := cli.ParseBuildOptions(flags)
@@ -218,12 +307,12 @@ func (service *serviceType) handleBuildRequest(id uint32, request map[string]int
 	// stdout there, the public Go API we're about to call doesn't have an option
 	// for "write to stdout but don't actually write" and I don't think it should.
 	// For now let's just forbid this case because it's not even that useful.
-	if err == nil && write && options.Outfile == "" && options.Outdir == "" {
+	if err == nil && !isServe && write && options.Outfile == "" && options.Outdir == "" {
 		err = errors.New("Either provide \"outfile\" or set \"write\" to false")
 	}
 
 	if err != nil {
-		return encodeErrorPacket(id, err)
+		return outgoingPacket{bytes: encodeErrorPacket(id, err)}
 	}
 
 	// Optionally allow input from the stdin channel
@@ -272,13 +361,13 @@ func (service *serviceType) handleBuildRequest(id uint32, request map[string]int
 			pluginName := p["name"].(string)
 
 			if callbacks, err := filteredCallbacks(pluginName, "onResolve", p["onResolve"].([]interface{})); err != nil {
-				return encodeErrorPacket(id, err)
+				return outgoingPacket{bytes: encodeErrorPacket(id, err)}
 			} else {
 				onResolveCallbacks = append(onResolveCallbacks, callbacks...)
 			}
 
 			if callbacks, err := filteredCallbacks(pluginName, "onLoad", p["onLoad"].([]interface{})); err != nil {
-				return encodeErrorPacket(id, err)
+				return outgoingPacket{bytes: encodeErrorPacket(id, err)}
 			} else {
 				onLoadCallbacks = append(onLoadCallbacks, callbacks...)
 			}
@@ -412,22 +501,123 @@ func (service *serviceType) handleBuildRequest(id uint32, request map[string]int
 		})
 	}
 
+	if isServe {
+		var serveOptions api.ServeOptions
+		serve := serve.(map[string]interface{})
+		serveID := serve["serveID"].(int)
+		if port, ok := serve["port"]; ok {
+			serveOptions.Port = uint16(port.(int))
+		}
+		serveOptions.OnRequest = func(args api.ServeOnRequestArgs) {
+			service.sendRequest(map[string]interface{}{
+				"command": "serve-request",
+				"serveID": serveID,
+				"args": map[string]interface{}{
+					"remoteAddress": args.RemoteAddress,
+					"method":        args.Method,
+					"path":          args.Path,
+					"status":        args.Status,
+					"timeInMS":      args.TimeInMS,
+				},
+			})
+		}
+		result, err := api.Serve(serveOptions, options)
+		if err != nil {
+			return outgoingPacket{bytes: encodeErrorPacket(id, err)}
+		}
+		response := map[string]interface{}{
+			"port": int(result.Port),
+		}
+
+		// Asynchronously wait for the server to stop, then fulfil the "wait" promise
+		go func() {
+			request := map[string]interface{}{
+				"command": "serve-wait",
+				"serveID": serveID,
+			}
+			if err := result.Wait(); err != nil {
+				request["error"] = err.Error()
+			} else {
+				request["error"] = nil
+			}
+			service.sendRequest(request)
+
+			// Only mutate the map while inside a mutex
+			service.mutex.Lock()
+			defer service.mutex.Unlock()
+			delete(service.serveStops, serveID)
+		}()
+
+		func() {
+			// Only mutate the map while inside a mutex
+			service.mutex.Lock()
+			defer service.mutex.Unlock()
+			service.serveStops[serveID] = result.Stop
+		}()
+
+		return outgoingPacket{
+			bytes: encodePacket(packet{
+				id:    id,
+				value: response,
+			}),
+
+			// Make sure the serve doesn't finish until "stop" has been called
+			refCount: 1,
+		}
+	}
+
+	rebuildID := service.nextRebuildID
+	if incremental {
+		service.nextRebuildID++
+	}
+
+	resultToResponse := func(result api.BuildResult) map[string]interface{} {
+		response := map[string]interface{}{
+			"errors":   encodeMessages(result.Errors),
+			"warnings": encodeMessages(result.Warnings),
+		}
+		if !write {
+			// Pass the output files back to the caller
+			response["outputFiles"] = encodeOutputFiles(result.OutputFiles)
+		}
+		if incremental {
+			response["rebuildID"] = rebuildID
+		}
+		return response
+	}
+
 	options.Write = write
+	options.Incremental = incremental
 	result := api.Build(options)
-	response := map[string]interface{}{
-		"errors":   encodeMessages(result.Errors),
-		"warnings": encodeMessages(result.Warnings),
+	response := resultToResponse(result)
+	refCount := 0
+
+	if incremental {
+		func() {
+			// Only mutate the map while inside a mutex
+			service.mutex.Lock()
+			defer service.mutex.Unlock()
+			service.rebuilds[rebuildID] = func(id uint32) []byte {
+				result := result.Rebuild()
+				response := resultToResponse(result)
+				return encodePacket(packet{
+					id:    id,
+					value: response,
+				})
+			}
+		}()
+
+		// Make sure the build doesn't finish until "dispose" has been called
+		refCount = 1
 	}
 
-	if !write {
-		// Pass the output files back to the caller
-		response["outputFiles"] = encodeOutputFiles(result.OutputFiles)
+	return outgoingPacket{
+		bytes: encodePacket(packet{
+			id:    id,
+			value: response,
+		}),
+		refCount: refCount,
 	}
-
-	return encodePacket(packet{
-		id:    id,
-		value: response,
-	})
 }
 
 func (service *serviceType) handleTransformRequest(id uint32, request map[string]interface{}) []byte {
@@ -566,7 +756,7 @@ func decodeMessages(values []interface{}) []api.Message {
 }
 
 func decodeMessageToPrivate(obj map[string]interface{}) logger.Msg {
-	msg := logger.Msg{Text: obj["text"].(string)}
+	msg := logger.Msg{Data: logger.MsgData{Text: obj["text"].(string)}}
 
 	// Some messages won't have a location
 	loc := obj["location"]
@@ -576,7 +766,7 @@ func decodeMessageToPrivate(obj map[string]interface{}) logger.Msg {
 		if namespace == "" {
 			namespace = "file"
 		}
-		msg.Location = &logger.MsgLocation{
+		msg.Data.Location = &logger.MsgLocation{
 			File:      loc["file"].(string),
 			Namespace: namespace,
 			Line:      loc["line"].(int),
