@@ -183,6 +183,11 @@ func TestPackageJsonSideEffectsFalseKeepBareImportAndRequireES6(t *testing.T) {
 			Mode:          config.ModeBundle,
 			AbsOutputFile: "/out.js",
 		},
+		expectedScanLog: `/Users/user/project/src/entry.js: warning: Ignoring this import because "` +
+			`/Users/user/project/node_modules/demo-pkg/index.js" was marked as having no side effects
+/Users/user/project/node_modules/demo-pkg/package.json: note: "sideEffects" is false ` +
+			`in the enclosing "package.json" file
+`,
 	})
 }
 
@@ -209,6 +214,11 @@ func TestPackageJsonSideEffectsFalseKeepBareImportAndRequireCommonJS(t *testing.
 			Mode:          config.ModeBundle,
 			AbsOutputFile: "/out.js",
 		},
+		expectedScanLog: `/Users/user/project/src/entry.js: warning: Ignoring this import because "` +
+			`/Users/user/project/node_modules/demo-pkg/index.js" was marked as having no side effects
+/Users/user/project/node_modules/demo-pkg/package.json: note: "sideEffects" is false ` +
+			`in the enclosing "package.json" file
+`,
 	})
 }
 
@@ -234,6 +244,11 @@ func TestPackageJsonSideEffectsFalseRemoveBareImportES6(t *testing.T) {
 			Mode:          config.ModeBundle,
 			AbsOutputFile: "/out.js",
 		},
+		expectedScanLog: `/Users/user/project/src/entry.js: warning: Ignoring this import because "` +
+			`/Users/user/project/node_modules/demo-pkg/index.js" was marked as having no side effects
+/Users/user/project/node_modules/demo-pkg/package.json: note: "sideEffects" is false ` +
+			`in the enclosing "package.json" file
+`,
 	})
 }
 
@@ -259,6 +274,11 @@ func TestPackageJsonSideEffectsFalseRemoveBareImportCommonJS(t *testing.T) {
 			Mode:          config.ModeBundle,
 			AbsOutputFile: "/out.js",
 		},
+		expectedScanLog: `/Users/user/project/src/entry.js: warning: Ignoring this import because "` +
+			`/Users/user/project/node_modules/demo-pkg/index.js" was marked as having no side effects
+/Users/user/project/node_modules/demo-pkg/package.json: note: "sideEffects" is false ` +
+			`in the enclosing "package.json" file
+`,
 	})
 }
 
@@ -412,6 +432,304 @@ func TestPackageJsonSideEffectsArrayKeep(t *testing.T) {
 	})
 }
 
+func TestPackageJsonSideEffectsArrayKeepMainUseModule(t *testing.T) {
+	dce_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/Users/user/project/src/entry.js": `
+				import {foo} from "demo-pkg"
+				console.log('unused import')
+			`,
+			"/Users/user/project/node_modules/demo-pkg/index-main.js": `
+				export const foo = 123
+				console.log('TEST FAILED')
+			`,
+			"/Users/user/project/node_modules/demo-pkg/index-module.js": `
+				export const foo = 123
+				console.log('TEST FAILED')
+			`,
+			"/Users/user/project/node_modules/demo-pkg/package.json": `
+				{
+					"main": "index-main.js",
+					"module": "index-module.js",
+					"sideEffects": ["./index-main.js"]
+				}
+			`,
+		},
+		entryPaths: []string{"/Users/user/project/src/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+			MainFields:    []string{"module"},
+		},
+	})
+}
+
+func TestPackageJsonSideEffectsArrayKeepMainUseMain(t *testing.T) {
+	dce_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/Users/user/project/src/entry.js": `
+				import {foo} from "demo-pkg"
+				console.log('unused import')
+			`,
+			"/Users/user/project/node_modules/demo-pkg/index-main.js": `
+				export const foo = 123
+				console.log('this should be kept')
+			`,
+			"/Users/user/project/node_modules/demo-pkg/index-module.js": `
+				export const foo = 123
+				console.log('TEST FAILED')
+			`,
+			"/Users/user/project/node_modules/demo-pkg/package.json": `
+				{
+					"main": "index-main.js",
+					"module": "index-module.js",
+					"sideEffects": ["./index-main.js"]
+				}
+			`,
+		},
+		entryPaths: []string{"/Users/user/project/src/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+			MainFields:    []string{"main"},
+		},
+	})
+}
+
+func TestPackageJsonSideEffectsArrayKeepMainImplicitModule(t *testing.T) {
+	dce_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/Users/user/project/src/entry.js": `
+				import {foo} from "demo-pkg"
+				console.log('unused import')
+			`,
+			"/Users/user/project/node_modules/demo-pkg/index-main.js": `
+				export const foo = 123
+				console.log('TEST FAILED')
+			`,
+			"/Users/user/project/node_modules/demo-pkg/index-module.js": `
+				export const foo = 123
+				console.log('TEST FAILED')
+			`,
+			"/Users/user/project/node_modules/demo-pkg/package.json": `
+				{
+					"main": "index-main.js",
+					"module": "index-module.js",
+					"sideEffects": ["./index-main.js"]
+				}
+			`,
+		},
+		entryPaths: []string{"/Users/user/project/src/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+		},
+	})
+}
+
+func TestPackageJsonSideEffectsArrayKeepMainImplicitMain(t *testing.T) {
+	dce_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/Users/user/project/src/entry.js": `
+				import {foo} from "demo-pkg"
+				import "./require-demo-pkg"
+				console.log('unused import')
+			`,
+			"/Users/user/project/src/require-demo-pkg.js": `
+				// This causes "index-main.js" to be selected
+				require('demo-pkg')
+			`,
+			"/Users/user/project/node_modules/demo-pkg/index-main.js": `
+				export const foo = 123
+				console.log('this should be kept')
+			`,
+			"/Users/user/project/node_modules/demo-pkg/index-module.js": `
+				export const foo = 123
+				console.log('TEST FAILED')
+			`,
+			"/Users/user/project/node_modules/demo-pkg/package.json": `
+				{
+					"main": "index-main.js",
+					"module": "index-module.js",
+					"sideEffects": ["./index-main.js"]
+				}
+			`,
+		},
+		entryPaths: []string{"/Users/user/project/src/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+		},
+	})
+}
+
+func TestPackageJsonSideEffectsArrayKeepModuleUseModule(t *testing.T) {
+	dce_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/Users/user/project/src/entry.js": `
+				import {foo} from "demo-pkg"
+				console.log('unused import')
+			`,
+			"/Users/user/project/node_modules/demo-pkg/index-main.js": `
+				export const foo = 123
+				console.log('TEST FAILED')
+			`,
+			"/Users/user/project/node_modules/demo-pkg/index-module.js": `
+				export const foo = 123
+				console.log('this should be kept')
+			`,
+			"/Users/user/project/node_modules/demo-pkg/package.json": `
+				{
+					"main": "index-main.js",
+					"module": "index-module.js",
+					"sideEffects": ["./index-module.js"]
+				}
+			`,
+		},
+		entryPaths: []string{"/Users/user/project/src/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+			MainFields:    []string{"module"},
+		},
+	})
+}
+
+func TestPackageJsonSideEffectsArrayKeepModuleUseMain(t *testing.T) {
+	dce_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/Users/user/project/src/entry.js": `
+				import {foo} from "demo-pkg"
+				console.log('unused import')
+			`,
+			"/Users/user/project/node_modules/demo-pkg/index-main.js": `
+				export const foo = 123
+				console.log('TEST FAILED')
+			`,
+			"/Users/user/project/node_modules/demo-pkg/index-module.js": `
+				export const foo = 123
+				console.log('TEST FAILED')
+			`,
+			"/Users/user/project/node_modules/demo-pkg/package.json": `
+				{
+					"main": "index-main.js",
+					"module": "index-module.js",
+					"sideEffects": ["./index-module.js"]
+				}
+			`,
+		},
+		entryPaths: []string{"/Users/user/project/src/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+			MainFields:    []string{"main"},
+		},
+	})
+}
+
+func TestPackageJsonSideEffectsArrayKeepModuleImplicitModule(t *testing.T) {
+	dce_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/Users/user/project/src/entry.js": `
+				import {foo} from "demo-pkg"
+				console.log('unused import')
+			`,
+			"/Users/user/project/node_modules/demo-pkg/index-main.js": `
+				export const foo = 123
+				console.log('TEST FAILED')
+			`,
+			"/Users/user/project/node_modules/demo-pkg/index-module.js": `
+				export const foo = 123
+				console.log('this should be kept')
+			`,
+			"/Users/user/project/node_modules/demo-pkg/package.json": `
+				{
+					"main": "index-main.js",
+					"module": "index-module.js",
+					"sideEffects": ["./index-module.js"]
+				}
+			`,
+		},
+		entryPaths: []string{"/Users/user/project/src/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+		},
+	})
+}
+
+func TestPackageJsonSideEffectsArrayKeepModuleImplicitMain(t *testing.T) {
+	dce_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/Users/user/project/src/entry.js": `
+				import {foo} from "demo-pkg"
+				import "./require-demo-pkg"
+				console.log('unused import')
+			`,
+			"/Users/user/project/src/require-demo-pkg.js": `
+				// This causes "index-main.js" to be selected
+				require('demo-pkg')
+			`,
+			"/Users/user/project/node_modules/demo-pkg/index-main.js": `
+				export const foo = 123
+				console.log('this should be kept')
+			`,
+			"/Users/user/project/node_modules/demo-pkg/index-module.js": `
+				export const foo = 123
+				console.log('TEST FAILED')
+			`,
+			"/Users/user/project/node_modules/demo-pkg/package.json": `
+				{
+					"main": "index-main.js",
+					"module": "index-module.js",
+					"sideEffects": ["./index-module.js"]
+				}
+			`,
+		},
+		entryPaths: []string{"/Users/user/project/src/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+		},
+	})
+}
+
+func TestPackageJsonSideEffectsArrayGlob(t *testing.T) {
+	dce_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/Users/user/project/src/entry.js": `
+				import "demo-pkg/keep/this/file"
+				import "demo-pkg/remove/this/file"
+			`,
+			"/Users/user/project/node_modules/demo-pkg/keep/this/file.js": `
+				console.log('this should be kept')
+			`,
+			"/Users/user/project/node_modules/demo-pkg/remove/this/file.js": `
+				console.log('TEST FAILED')
+			`,
+			"/Users/user/project/node_modules/demo-pkg/package.json": `
+				{
+					"sideEffects": [
+						"./ke?p/*/file.js",
+						"./remove/this/file.j",
+						"./re?ve/this/file.js"
+					]
+				}
+			`,
+		},
+		entryPaths: []string{"/Users/user/project/src/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+		},
+		expectedScanLog: `/Users/user/project/src/entry.js: warning: Ignoring this import because ` +
+			`"/Users/user/project/node_modules/demo-pkg/remove/this/file.js" was marked as having no side effects
+/Users/user/project/node_modules/demo-pkg/package.json: note: It was excluded from the "sideEffects" ` +
+			`array in the enclosing "package.json" file
+`,
+	})
+}
+
 func TestPackageJsonSideEffectsNestedDirectoryRemove(t *testing.T) {
 	dce_suite.expectBundled(t, bundled{
 		files: map[string]string{
@@ -662,3 +980,44 @@ func TestTreeShakingReactElements(t *testing.T) {
 		},
 	})
 }
+
+func TestDisableTreeShaking(t *testing.T) {
+	defines := config.ProcessDefines(map[string]config.DefineData{
+		"pure":    config.DefineData{CallCanBeUnwrappedIfUnused: true},
+		"some.fn": config.DefineData{CallCanBeUnwrappedIfUnused: true},
+	})
+	dce_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.jsx": `
+				import './remove-me'
+				function RemoveMe1() {}
+				let removeMe2 = 0
+				class RemoveMe3 {}
+
+				import './keep-me'
+				function KeepMe1() {}
+				let keepMe2 = <KeepMe1/>
+				function keepMe3() { console.log('side effects') }
+				let keepMe4 = /* @__PURE__ */ keepMe3()
+				let keepMe5 = pure()
+				let keepMe6 = some.fn()
+			`,
+			"/remove-me.js": `
+				export default 'unused'
+			`,
+			"/keep-me/index.js": `
+				console.log('side effects')
+			`,
+			"/keep-me/package.json": `
+				{ "sideEffects": false }
+			`,
+		},
+		entryPaths: []string{"/entry.jsx"},
+		options: config.Options{
+			Mode:                 config.ModeBundle,
+			AbsOutputFile:        "/out.js",
+			IgnoreDCEAnnotations: true,
+			Defines:              &defines,
+		},
+	})
+}
