@@ -229,7 +229,7 @@ func TestExportFormsIIFE(t *testing.T) {
 		options: config.Options{
 			Mode:          config.ModeBundle,
 			OutputFormat:  config.FormatIIFE,
-			ModuleName:    []string{"moduleName"},
+			GlobalName:    []string{"globalName"},
 			AbsOutputFile: "/out.js",
 		},
 	})
@@ -841,6 +841,155 @@ func TestRequireAndDynamicImportInvalidTemplate(t *testing.T) {
 	})
 }
 
+func TestDynamicImportWithExpressionCJS(t *testing.T) {
+	default_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/a.js": `
+				import('foo')
+				import(foo())
+			`,
+		},
+		entryPaths: []string{"/a.js"},
+		options: config.Options{
+			Mode:          config.ModeConvertFormat,
+			OutputFormat:  config.FormatCommonJS,
+			AbsOutputFile: "/out.js",
+		},
+	})
+}
+
+func TestDynamicImportWithExpressionCJSAndES5(t *testing.T) {
+	default_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/a.js": `
+				import('foo')
+				import(foo())
+			`,
+		},
+		entryPaths: []string{"/a.js"},
+		options: config.Options{
+			Mode:                  config.ModeConvertFormat,
+			OutputFormat:          config.FormatCommonJS,
+			UnsupportedJSFeatures: es(5),
+			AbsOutputFile:         "/out.js",
+		},
+	})
+}
+
+func TestMinifiedDynamicImportWithExpressionCJS(t *testing.T) {
+	default_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/a.js": `
+				import('foo')
+				import(foo())
+			`,
+		},
+		entryPaths: []string{"/a.js"},
+		options: config.Options{
+			Mode:             config.ModeConvertFormat,
+			OutputFormat:     config.FormatCommonJS,
+			AbsOutputFile:    "/out.js",
+			RemoveWhitespace: true,
+		},
+	})
+}
+
+func TestMinifiedDynamicImportWithExpressionCJSAndES5(t *testing.T) {
+	default_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/a.js": `
+				import('foo')
+				import(foo())
+			`,
+		},
+		entryPaths: []string{"/a.js"},
+		options: config.Options{
+			Mode:                  config.ModeConvertFormat,
+			OutputFormat:          config.FormatCommonJS,
+			UnsupportedJSFeatures: es(5),
+			AbsOutputFile:         "/out.js",
+			RemoveWhitespace:      true,
+		},
+	})
+}
+
+func TestConditionalRequireResolve(t *testing.T) {
+	default_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/a.js": `
+				require.resolve(x ? 'a' : y ? 'b' : 'c')
+				require.resolve(x ? y ? 'a' : 'b' : c)
+			`,
+		},
+		entryPaths: []string{"/a.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+			ExternalModules: config.ExternalModules{
+				NodeModules: map[string]bool{
+					"a": true,
+					"b": true,
+					"c": true,
+				},
+			},
+		},
+	})
+}
+
+func TestConditionalRequire(t *testing.T) {
+	default_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/a.js": `
+				require(x ? 'a' : y ? './b' : 'c')
+				require(x ? y ? 'a' : './b' : c)
+			`,
+			"/b.js": `
+				exports.foo = 213
+			`,
+		},
+		entryPaths: []string{"/a.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+			ExternalModules: config.ExternalModules{
+				NodeModules: map[string]bool{
+					"a": true,
+					"c": true,
+				},
+			},
+		},
+		expectedScanLog: `/a.js: warning: This call to "require" will not be bundled because the argument is not a string literal
+`,
+	})
+}
+
+func TestConditionalImport(t *testing.T) {
+	default_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/a.js": `
+				import(x ? 'a' : y ? './b' : 'c')
+				import(x ? y ? 'a' : './b' : c)
+			`,
+			"/b.js": `
+				exports.foo = 213
+			`,
+		},
+		entryPaths: []string{"/a.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+			ExternalModules: config.ExternalModules{
+				NodeModules: map[string]bool{
+					"a": true,
+					"c": true,
+				},
+			},
+		},
+		expectedScanLog: `/a.js: warning: This dynamic import will not be bundled because the argument is not a string literal
+`,
+	})
+}
+
 func TestRequireBadArgumentCount(t *testing.T) {
 	default_suite.expectBundled(t, bundled{
 		files: map[string]string{
@@ -2011,6 +2160,41 @@ func TestAutoExternal(t *testing.T) {
 	})
 }
 
+func TestExternalWithWildcard(t *testing.T) {
+	default_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				// Should match
+				import "/assets/images/test.jpg";
+				import "/dir/x/file.gif";
+				import "/dir//file.gif";
+				import "./file.png";
+
+				// Should not match
+				import "/sassets/images/test.jpg";
+				import "/dir/file.gif";
+				import "./file.ping";
+			`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:         config.ModeBundle,
+			AbsOutputDir: "/out",
+			ExternalModules: config.ExternalModules{
+				Patterns: []config.WildcardPattern{
+					{Prefix: "/assets/"},
+					{Suffix: ".png"},
+					{Prefix: "/dir/", Suffix: "/file.gif"},
+				},
+			},
+		},
+		expectedScanLog: `/entry.js: error: Could not read from file: /sassets/images/test.jpg
+/entry.js: error: Could not read from file: /dir/file.gif
+/entry.js: error: Could not resolve "./file.ping"
+`,
+	})
+}
+
 // This test case makes sure many entry points don't cause a crash
 func TestManyEntryPoints(t *testing.T) {
 	default_suite.expectBundled(t, bundled{
@@ -2605,9 +2789,9 @@ func TestOutputExtensionRemappingFile(t *testing.T) {
 		},
 		entryPaths: []string{"/entry.js"},
 		options: config.Options{
-			Mode:             config.ModeBundle,
-			OutputExtensions: map[string]string{".js": ".notjs"},
-			AbsOutputFile:    "/out.js",
+			Mode:              config.ModeBundle,
+			OutputExtensionJS: ".notjs",
+			AbsOutputFile:     "/out.js",
 		},
 	})
 }
@@ -2621,9 +2805,9 @@ func TestOutputExtensionRemappingDir(t *testing.T) {
 		},
 		entryPaths: []string{"/entry.js"},
 		options: config.Options{
-			Mode:             config.ModeBundle,
-			OutputExtensions: map[string]string{".js": ".notjs"},
-			AbsOutputDir:     "/out",
+			Mode:              config.ModeBundle,
+			OutputExtensionJS: ".notjs",
+			AbsOutputDir:      "/out",
 		},
 	})
 }
@@ -2941,7 +3125,6 @@ func TestRequireResolve(t *testing.T) {
 		expectedScanLog: `/entry.js: warning: Indirect calls to "require" will not be bundled (surround with a try/catch to silence this warning)
 /entry.js: warning: Indirect calls to "require" will not be bundled (surround with a try/catch to silence this warning)
 /entry.js: warning: Indirect calls to "require" will not be bundled (surround with a try/catch to silence this warning)
-/entry.js: warning: Indirect calls to "require" will not be bundled (surround with a try/catch to silence this warning)
 /entry.js: warning: "./present-file" should be marked as external for use with "require.resolve"
 /entry.js: warning: "./missing-file" should be marked as external for use with "require.resolve"
 /entry.js: warning: "missing-pkg" should be marked as external for use with "require.resolve"
@@ -3226,6 +3409,27 @@ func TestOutbase(t *testing.T) {
 	})
 }
 
+func TestAvoidTDZ(t *testing.T) {
+	default_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				class Foo {
+					static foo = new Foo
+				}
+				let foo = Foo.foo
+				console.log(foo)
+				export class Bar {}
+				export let bar = 123
+			`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+		},
+	})
+}
+
 func TestAvoidTDZNoBundle(t *testing.T) {
 	default_suite.expectBundled(t, bundled{
 		files: map[string]string{
@@ -3238,14 +3442,11 @@ func TestAvoidTDZNoBundle(t *testing.T) {
 				export class Bar {}
 				export let bar = 123
 			`,
-			"/foo.js": `
-			`,
 		},
 		entryPaths: []string{"/entry.js"},
 		options: config.Options{
 			Mode:          config.ModePassThrough,
 			AbsOutputFile: "/out.js",
-			AvoidTDZ:      true,
 		},
 	})
 }
@@ -3354,3 +3555,131 @@ func TestKeepNamesTreeShaking(t *testing.T) {
 		},
 	})
 }
+
+func TestCharFreqIgnoreComments(t *testing.T) {
+	default_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/a.js": `
+				export default function(one, two, three, four) {
+					return 'the argument names must be the same'
+				}
+			`,
+			"/b.js": `
+				export default function(one, two, three, four) {
+					return 'the argument names must be the same'
+				}
+
+				// Some comment text to change the character frequency histogram:
+				// ________________________________________________________________________________
+				// FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
+				// AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
+				// IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
+				// LLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLL
+			`,
+		},
+		entryPaths: []string{"/a.js", "/b.js"},
+		options: config.Options{
+			Mode:              config.ModeBundle,
+			AbsOutputDir:      "/out",
+			MinifyIdentifiers: true,
+		},
+	})
+}
+
+func TestImportRelativeAsPackage(t *testing.T) {
+	default_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/Users/user/project/src/entry.js": `
+				import 'some/other/file'
+			`,
+			"/Users/user/project/src/some/other/file.js": `
+			`,
+		},
+		entryPaths: []string{"/Users/user/project/src/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/Users/user/project/out.js",
+		},
+		expectedScanLog: `/Users/user/project/src/entry.js: error: Could not resolve "some/other/file" ` +
+			`(use "./some/other/file" to import "/Users/user/project/src/some/other/file.js")
+`,
+	})
+}
+
+func TestForbidConstAssignWhenBundling(t *testing.T) {
+	default_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				const x = 1
+				x = 2
+			`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+		},
+		expectedScanLog: `/entry.js: error: Cannot assign to "x" because it is a constant
+/entry.js: note: "x" was declared a constant here
+`,
+	})
+}
+
+func TestConstWithLet(t *testing.T) {
+	default_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				const a = 1; console.log(a)
+				if (true) { const b = 2; console.log(b) }
+				for (const c = x;;) console.log(c)
+				for (const d in x) console.log(d)
+				for (const e of x) console.log(e)
+			`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+			MangleSyntax:  true,
+		},
+	})
+}
+
+func TestConstWithLetNoBundle(t *testing.T) {
+	default_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				const a = 1; console.log(a)
+				if (true) { const b = 2; console.log(b) }
+				for (const c = x;;) console.log(c)
+				for (const d in x) console.log(d)
+				for (const e of x) console.log(e)
+			`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:          config.ModePassThrough,
+			AbsOutputFile: "/out.js",
+			MangleSyntax:  true,
+		},
+	})
+}
+
+func TestConstWithLetNoMangle(t *testing.T) {
+	default_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				const a = 1; console.log(a)
+				if (true) { const b = 2; console.log(b) }
+				for (const c = x;;) console.log(c)
+				for (const d in x) console.log(d)
+				for (const e of x) console.log(e)
+			`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+		},
+	})
+}
