@@ -89,9 +89,9 @@ const (
 	//     ... bundled code ...
 	//   })();
 	//
-	// If the optional ModuleName is configured, then we'll write out this:
+	// If the optional GlobalName is configured, then we'll write out this:
 	//
-	//   let moduleName = (() => {
+	//   let globalName = (() => {
 	//     ... bundled code ...
 	//     return exports;
 	//   })();
@@ -136,9 +136,15 @@ type StdinInfo struct {
 	AbsResolveDir string
 }
 
+type WildcardPattern struct {
+	Prefix string
+	Suffix string
+}
+
 type ExternalModules struct {
 	NodeModules map[string]bool
 	AbsPaths    map[string]bool
+	Patterns    []WildcardPattern
 }
 
 type Mode uint8
@@ -174,9 +180,9 @@ type Options struct {
 	OmitRuntimeForTests     bool
 	PreserveUnusedImportsTS bool
 	UseDefineForClassFields bool
-	AvoidTDZ                bool
 	ASCIIOnly               bool
 	KeepNames               bool
+	IgnoreDCEAnnotations    bool
 
 	Defines  *ProcessedDefines
 	TS       TSOptions
@@ -190,17 +196,20 @@ type Options struct {
 	MainFields      []string
 	ExternalModules ExternalModules
 
-	AbsOutputFile     string
-	AbsOutputDir      string
-	AbsOutputBase     string
-	OutputExtensions  map[string]string
-	ModuleName        []string
-	TsConfigOverride  string
-	ExtensionToLoader map[string]Loader
-	OutputFormat      Format
-	PublicPath        string
-	InjectAbsPaths    []string
-	InjectedFiles     []InjectedFile
+	AbsOutputFile      string
+	AbsOutputDir       string
+	AbsOutputBase      string
+	OutputExtensionJS  string
+	OutputExtensionCSS string
+	GlobalName         []string
+	TsConfigOverride   string
+	ExtensionToLoader  map[string]Loader
+	OutputFormat       Format
+	PublicPath         string
+	InjectAbsPaths     []string
+	InjectedFiles      []InjectedFile
+	Banner             string
+	Footer             string
 
 	Plugins []Plugin
 
@@ -217,13 +226,6 @@ type InjectedFile struct {
 	Exports     []string
 }
 
-func (options *Options) OutputExtensionFor(key string) string {
-	if ext, ok := options.OutputExtensions[key]; ok {
-		return ext
-	}
-	return key
-}
-
 var filterMutex sync.Mutex
 var filterCache map[string]*regexp.Regexp
 
