@@ -17,15 +17,15 @@ import (
 func (p *parser) markSyntaxFeature(feature compat.JSFeature, r logger.Range) (didGenerateError bool) {
 	didGenerateError = true
 
-	if !p.UnsupportedJSFeatures.Has(feature) {
+	if !p.options.unsupportedJSFeatures.Has(feature) {
 		if feature == compat.TopLevelAwait {
-			if p.Mode == config.ModeBundle {
+			if p.options.mode == config.ModeBundle {
 				p.log.AddRangeError(&p.source, r, "Top-level await is currently not supported when bundling")
 				return
 			}
-			if p.Mode == config.ModeConvertFormat && !p.OutputFormat.KeepES6ImportExportSyntax() {
+			if p.options.mode == config.ModeConvertFormat && !p.options.outputFormat.KeepES6ImportExportSyntax() {
 				p.log.AddRangeError(&p.source, r, fmt.Sprintf(
-					"Top-level await is currently not supported with the %q output format", p.OutputFormat.String()))
+					"Top-level await is currently not supported with the %q output format", p.options.outputFormat.String()))
 				return
 			}
 		}
@@ -120,13 +120,13 @@ func (p *parser) markSyntaxFeature(feature compat.JSFeature, r logger.Range) (di
 // Mark the feature if "loweredFeature" is unsupported. This is used when one
 // feature is implemented in terms of another feature.
 func (p *parser) markLoweredSyntaxFeature(feature compat.JSFeature, r logger.Range, loweredFeature compat.JSFeature) {
-	if p.UnsupportedJSFeatures.Has(loweredFeature) {
+	if p.options.unsupportedJSFeatures.Has(loweredFeature) {
 		p.markSyntaxFeature(feature, r)
 	}
 }
 
 func (p *parser) isPrivateUnsupported(private *js_ast.EPrivateIdentifier) bool {
-	return p.UnsupportedJSFeatures.Has(p.symbols[private.Ref.InnerIndex].Kind.Feature())
+	return p.options.unsupportedJSFeatures.Has(p.symbols[private.Ref.InnerIndex].Kind.Feature())
 }
 
 func (p *parser) captureThis() js_ast.Ref {
@@ -155,7 +155,7 @@ func (p *parser) lowerFunction(
 	isArrow bool,
 ) {
 	// Lower object rest binding patterns in function arguments
-	if p.UnsupportedJSFeatures.Has(compat.ObjectRestSpread) {
+	if p.options.unsupportedJSFeatures.Has(compat.ObjectRestSpread) {
 		var prefixStmts []js_ast.Stmt
 
 		// Lower each argument individually instead of lowering all arguments
@@ -212,7 +212,7 @@ func (p *parser) lowerFunction(
 	}
 
 	// Lower async functions
-	if p.UnsupportedJSFeatures.Has(compat.AsyncAwait) && *isAsync {
+	if p.options.unsupportedJSFeatures.Has(compat.AsyncAwait) && *isAsync {
 		// Use the shortened form if we're an arrow function
 		if preferExpr != nil {
 			*preferExpr = true
@@ -449,7 +449,7 @@ flatten:
 	// Don't lower this if we don't need to. This check must be done here instead
 	// of earlier so we can do the dead code elimination above when the target is
 	// null or undefined.
-	if !p.UnsupportedJSFeatures.Has(compat.OptionalChain) && !containsPrivateName {
+	if !p.options.unsupportedJSFeatures.Has(compat.OptionalChain) && !containsPrivateName {
 		return originalExpr, exprOut{}
 	}
 
@@ -722,7 +722,7 @@ func (p *parser) lowerExponentiationAssignmentOperator(loc logger.Loc, e *js_ast
 
 func (p *parser) lowerNullishCoalescingAssignmentOperator(loc logger.Loc, e *js_ast.EBinary) js_ast.Expr {
 	if target, privateLoc, private := p.extractPrivateIndex(e.Left); private != nil {
-		if p.UnsupportedJSFeatures.Has(compat.NullishCoalescing) {
+		if p.options.unsupportedJSFeatures.Has(compat.NullishCoalescing) {
 			// "a.#b ??= c" => "(_a = __privateGet(a, #b)) != null ? _a : __privateSet(a, #b, c)"
 			targetFunc, targetWrapFunc := p.captureValueWithPossibleSideEffects(loc, 2, target)
 			left := p.lowerPrivateGet(targetFunc(), privateLoc, private)
@@ -740,7 +740,7 @@ func (p *parser) lowerNullishCoalescingAssignmentOperator(loc logger.Loc, e *js_
 	}
 
 	return p.lowerAssignmentOperator(e.Left, func(a js_ast.Expr, b js_ast.Expr) js_ast.Expr {
-		if p.UnsupportedJSFeatures.Has(compat.NullishCoalescing) {
+		if p.options.unsupportedJSFeatures.Has(compat.NullishCoalescing) {
 			// "a ??= b" => "(_a = a) != null ? _a : a = b"
 			return p.lowerNullishCoalescing(loc, a, js_ast.Assign(b, e.Right))
 		}
@@ -819,7 +819,7 @@ func (p *parser) lowerNullishCoalescing(loc logger.Loc, left js_ast.Expr, right
 func (p *parser) lowerObjectSpread(loc logger.Loc, e *js_ast.EObject) js_ast.Expr {
 	needsLowering := false
 
-	if p.UnsupportedJSFeatures.Has(compat.ObjectRestSpread) {
+	if p.options.unsupportedJSFeatures.Has(compat.ObjectRestSpread) {
 		for _, property := range e.Properties {
 			if property.Kind == js_ast.PropertySpread {
 				needsLowering = true
@@ -1028,7 +1028,7 @@ func exprHasObjectRest(expr js_ast.Expr) bool {
 }
 
 func (p *parser) lowerObjectRestInDecls(decls []js_ast.Decl) []js_ast.Decl {
-	if !p.UnsupportedJSFeatures.Has(compat.ObjectRestSpread) {
+	if !p.options.unsupportedJSFeatures.Has(compat.ObjectRestSpread) {
 		return decls
 	}
 
@@ -1056,7 +1056,7 @@ func (p *parser) lowerObjectRestInDecls(decls []js_ast.Decl) []js_ast.Decl {
 }
 
 func (p *parser) lowerObjectRestInForLoopInit(init js_ast.Stmt, body *js_ast.Stmt) {
-	if !p.UnsupportedJSFeatures.Has(compat.ObjectRestSpread) {
+	if !p.options.unsupportedJSFeatures.Has(compat.ObjectRestSpread) {
 		return
 	}
 
@@ -1100,7 +1100,7 @@ func (p *parser) lowerObjectRestInForLoopInit(init js_ast.Stmt, body *js_ast.Stm
 }
 
 func (p *parser) lowerObjectRestInCatchBinding(catch *js_ast.Catch) {
-	if !p.UnsupportedJSFeatures.Has(compat.ObjectRestSpread) {
+	if !p.options.unsupportedJSFeatures.Has(compat.ObjectRestSpread) {
 		return
 	}
 
@@ -1152,7 +1152,7 @@ func (p *parser) lowerObjectRestHelper(
 	assign func(js_ast.Expr, js_ast.Expr),
 	declare generateTempRefArg,
 ) bool {
-	if !p.UnsupportedJSFeatures.Has(compat.ObjectRestSpread) {
+	if !p.options.unsupportedJSFeatures.Has(compat.ObjectRestSpread) {
 		return false
 	}
 
@@ -1441,7 +1441,7 @@ func (p *parser) captureKeyForObjectRest(originalKey js_ast.Expr) (finalKey js_a
 
 // Lower class fields for environments that don't support them. This either
 // takes a statement or an expression.
-func (p *parser) lowerClass(stmt js_ast.Stmt, expr js_ast.Expr) ([]js_ast.Stmt, js_ast.Expr) {
+func (p *parser) lowerClass(stmt js_ast.Stmt, expr js_ast.Expr, shadowRef js_ast.Ref) ([]js_ast.Stmt, js_ast.Expr) {
 	type classKind uint8
 	const (
 		classKindExpr classKind = iota
@@ -1455,10 +1455,27 @@ func (p *parser) lowerClass(stmt js_ast.Stmt, expr js_ast.Expr) ([]js_ast.Stmt,
 	var class *js_ast.Class
 	var classLoc logger.Loc
 	var defaultName js_ast.LocRef
+	var nameToKeep string
 	if stmt.Data == nil {
 		e, _ := expr.Data.(*js_ast.EClass)
 		class = &e.Class
 		kind = classKindExpr
+		if class.Name != nil {
+			symbol := &p.symbols[class.Name.Ref.InnerIndex]
+			nameToKeep = symbol.OriginalName
+
+			// The shadowing name inside the class expression should be the same as
+			// the class expression name itself
+			if shadowRef != js_ast.InvalidRef {
+				p.mergeSymbols(shadowRef, class.Name.Ref)
+			}
+
+			// Remove unused class names when minifying. Check this after we merge in
+			// the shadowing name above since that will adjust the use count.
+			if p.options.mangleSyntax && symbol.UseCountEstimate == 0 {
+				class.Name = nil
+			}
+		}
 	} else if s, ok := stmt.Data.(*js_ast.SClass); ok {
 		class = &s.Class
 		if s.IsExport {
@@ -1466,26 +1483,17 @@ func (p *parser) lowerClass(stmt js_ast.Stmt, expr js_ast.Expr) ([]js_ast.Stmt,
 		} else {
 			kind = classKindStmt
 		}
+		nameToKeep = p.symbols[class.Name.Ref.InnerIndex].OriginalName
 	} else {
 		s, _ := stmt.Data.(*js_ast.SExportDefault)
 		s2, _ := s.Value.Stmt.Data.(*js_ast.SClass)
 		class = &s2.Class
 		defaultName = s.DefaultName
 		kind = classKindExportDefaultStmt
-	}
-
-	// We always lower class fields when parsing TypeScript since class fields in
-	// TypeScript don't follow the JavaScript spec. We also need to always lower
-	// TypeScript-style decorators since they don't have a JavaScript equivalent.
-	classFeatures := compat.ClassField | compat.ClassStaticField |
-		compat.ClassPrivateField | compat.ClassPrivateStaticField |
-		compat.ClassPrivateMethod | compat.ClassPrivateStaticMethod |
-		compat.ClassPrivateAccessor | compat.ClassPrivateStaticAccessor
-	if !p.TS.Parse && !p.UnsupportedJSFeatures.Has(classFeatures) {
-		if kind == classKindExpr {
-			return nil, expr
+		if class.Name != nil {
+			nameToKeep = p.symbols[class.Name.Ref.InnerIndex].OriginalName
 		} else {
-			return []js_ast.Stmt{stmt}, js_ast.Expr{}
+			nameToKeep = "default"
 		}
 	}
 
@@ -1543,7 +1551,7 @@ func (p *parser) lowerClass(stmt js_ast.Stmt, expr js_ast.Expr) ([]js_ast.Stmt,
 			//   }, _a.foo = 123, _a.bar = _a.foo, _a);
 			//
 			if class.Name != nil {
-				p.symbols[class.Name.Ref.InnerIndex].Link = name.Data.(*js_ast.EIdentifier).Ref
+				p.mergeSymbols(class.Name.Ref, name.Data.(*js_ast.EIdentifier).Ref)
 				class.Name = nil
 			}
 
@@ -1553,7 +1561,7 @@ func (p *parser) lowerClass(stmt js_ast.Stmt, expr js_ast.Expr) ([]js_ast.Stmt,
 				if kind == classKindExportDefaultStmt {
 					class.Name = &defaultName
 				} else {
-					class.Name = &js_ast.LocRef{Loc: classLoc, Ref: p.generateTempRef(tempRefNoDeclare, "")}
+					class.Name = &js_ast.LocRef{Loc: classLoc, Ref: p.generateTempRef(tempRefNoDeclare, "zomzomz")}
 				}
 			}
 			p.recordUsage(class.Name.Ref)
@@ -1563,7 +1571,7 @@ func (p *parser) lowerClass(stmt js_ast.Stmt, expr js_ast.Expr) ([]js_ast.Stmt,
 
 	for _, prop := range class.Properties {
 		// Merge parameter decorators with method decorators
-		if p.TS.Parse && prop.IsMethod {
+		if p.options.ts.Parse && prop.IsMethod {
 			if fn, ok := prop.Value.Data.(*js_ast.EFunction); ok {
 				for i, arg := range fn.Fn.Args {
 					for _, decorator := range arg.TSDecorators {
@@ -1586,17 +1594,17 @@ func (p *parser) lowerClass(stmt js_ast.Stmt, expr js_ast.Expr) ([]js_ast.Stmt,
 		// strict class field initialization, so we shouldn't either.
 		private, _ := prop.Key.Data.(*js_ast.EPrivateIdentifier)
 		mustLowerPrivate := private != nil && p.isPrivateUnsupported(private)
-		shouldOmitFieldInitializer := p.TS.Parse && !prop.IsMethod && prop.Initializer == nil &&
-			!p.UseDefineForClassFields && !mustLowerPrivate
+		shouldOmitFieldInitializer := p.options.ts.Parse && !prop.IsMethod && prop.Initializer == nil &&
+			!p.options.useDefineForClassFields && !mustLowerPrivate
 
 		// Class fields must be lowered if the environment doesn't support them
-		mustLowerField := !prop.IsMethod && (!prop.IsStatic && p.UnsupportedJSFeatures.Has(compat.ClassField) ||
-			(prop.IsStatic && p.UnsupportedJSFeatures.Has(compat.ClassStaticField)))
+		mustLowerField := !prop.IsMethod && (!prop.IsStatic && p.options.unsupportedJSFeatures.Has(compat.ClassField) ||
+			(prop.IsStatic && p.options.unsupportedJSFeatures.Has(compat.ClassStaticField)))
 
 		// Make sure the order of computed property keys doesn't change. These
 		// expressions have side effects and must be evaluated in order.
 		keyExprNoSideEffects := prop.Key
-		if prop.IsComputed && (p.TS.Parse || len(prop.TSDecorators) > 0 ||
+		if prop.IsComputed && (p.options.ts.Parse || len(prop.TSDecorators) > 0 ||
 			mustLowerField || computedPropertyCache.Data != nil) {
 			needsKey := true
 			if len(prop.TSDecorators) == 0 && (prop.IsMethod || shouldOmitFieldInitializer) {
@@ -1625,7 +1633,7 @@ func (p *parser) lowerClass(stmt js_ast.Stmt, expr js_ast.Expr) ([]js_ast.Stmt,
 		}
 
 		// Handle decorators
-		if p.TS.Parse {
+		if p.options.ts.Parse {
 			// Generate a single call to "__decorate()" for this property
 			if len(prop.TSDecorators) > 0 {
 				loc := prop.Key.Loc
@@ -1677,7 +1685,7 @@ func (p *parser) lowerClass(stmt js_ast.Stmt, expr js_ast.Expr) ([]js_ast.Stmt,
 		// TypeScript feature. Move their initializers from the class body to
 		// either the constructor (instance fields) or after the class (static
 		// fields).
-		if !prop.IsMethod && (mustLowerField || (p.TS.Parse && !p.UseDefineForClassFields && (!prop.IsStatic || private == nil))) {
+		if !prop.IsMethod && (mustLowerField || (p.options.ts.Parse && !p.options.useDefineForClassFields && (!prop.IsStatic || private == nil))) {
 			// The TypeScript compiler doesn't follow the JavaScript spec for
 			// uninitialized fields. They are supposed to be set to undefined but the
 			// TypeScript compiler just omits them entirely.
@@ -1731,7 +1739,7 @@ func (p *parser) lowerClass(stmt js_ast.Stmt, expr js_ast.Expr) ([]js_ast.Stmt,
 						},
 					}}
 					p.recordUsage(ref)
-				} else if private == nil && p.UseDefineForClassFields {
+				} else if private == nil && p.options.useDefineForClassFields {
 					if _, ok := init.Data.(*js_ast.EUndefined); ok {
 						expr = p.callRuntime(loc, "__publicField", []js_ast.Expr{target, prop.Key})
 					} else {
@@ -1841,7 +1849,7 @@ func (p *parser) lowerClass(stmt js_ast.Stmt, expr js_ast.Expr) ([]js_ast.Stmt,
 					ctor = fn
 
 					// Initialize TypeScript constructor parameter fields
-					if p.TS.Parse {
+					if p.options.ts.Parse {
 						for _, arg := range ctor.Fn.Args {
 							if arg.IsTypeScriptCtorField {
 								if id, ok := arg.Binding.Data.(*js_ast.BIdentifier); ok {
@@ -1946,23 +1954,114 @@ func (p *parser) lowerClass(stmt js_ast.Stmt, expr js_ast.Expr) ([]js_ast.Stmt,
 		if wrapFunc != nil {
 			expr = wrapFunc(expr)
 		}
+
+		// Optionally preserve the name
+		if p.options.keepNames && nameToKeep != "" {
+			expr = p.keepExprSymbolName(expr, nameToKeep)
+		}
 		return nil, expr
 	}
 
+	// If this is true, we have removed some code from the class body that could
+	// potentially contain an expression that captures the shadowing class name.
+	// This could lead to incorrect behavior if the class is later re-assigned,
+	// since the removed code would no longer be in the shadowing scope.
+	hasPotentialShadowCaptureEscape := shadowRef != js_ast.InvalidRef &&
+		(computedPropertyCache.Data != nil ||
+			len(privateMembers) > 0 ||
+			len(staticMembers) > 0 ||
+			len(instanceDecorators) > 0 ||
+			len(staticDecorators) > 0 ||
+			len(class.TSDecorators) > 0)
+
+	// Optionally preserve the name
+	var keepNameStmt js_ast.Stmt
+	if p.options.keepNames && nameToKeep != "" {
+		name := nameFunc()
+		keepNameStmt = p.keepStmtSymbolName(name.Loc, name.Data.(*js_ast.EIdentifier).Ref, nameToKeep)
+	}
+
+	// Safari workaround: Automatically avoid TDZ issues when bundling
+	avoidTDZ := p.options.mode == config.ModeBundle && p.currentScope.Parent == nil
+
 	// Pack the class back into a statement, with potentially some extra
 	// statements afterwards
 	var stmts []js_ast.Stmt
-	if len(class.TSDecorators) > 0 {
+	var nameForClassDecorators js_ast.LocRef
+	generatedLocalStmt := false
+	if len(class.TSDecorators) > 0 || hasPotentialShadowCaptureEscape || avoidTDZ {
+		generatedLocalStmt = true
 		name := nameFunc()
-		id, _ := name.Data.(*js_ast.EIdentifier)
+		nameRef := name.Data.(*js_ast.EIdentifier).Ref
+		nameForClassDecorators = js_ast.LocRef{Loc: name.Loc, Ref: nameRef}
 		classExpr := js_ast.EClass{Class: *class}
 		class = &classExpr.Class
+		init := &js_ast.Expr{Loc: classLoc, Data: &classExpr}
+
+		if hasPotentialShadowCaptureEscape && len(class.TSDecorators) == 0 {
+			// If something captures the shadowing name and escapes the class body,
+			// make a new constant to store the class and forward that value to a
+			// mutable alias. That way if the alias is mutated, everything bound to
+			// the original constant doesn't change.
+			//
+			//   class Foo {
+			//     static foo() { return this.#foo() }
+			//     static #foo() { return Foo }
+			//   }
+			//   Foo = class Bar {}
+			//
+			// becomes:
+			//
+			//   var _foo, foo_fn;
+			//   const Foo2 = class {
+			//     static foo() {
+			//       return __privateMethod(this, _foo, foo_fn).call(this);
+			//     }
+			//   };
+			//   let Foo = Foo2;
+			//   _foo = new WeakSet();
+			//   foo_fn = function() {
+			//     return Foo2;
+			//   };
+			//   _foo.add(Foo);
+			//   Foo = class Bar {
+			//   };
+			//
+			// Generate a new symbol instead of using the shadowing name directly
+			// because the shadowing name isn't a top-level symbol and we are now
+			// making a top-level symbol. This symbol must be minified along with
+			// other top-level symbols to avoid name collisions.
+			captureRef := p.newSymbol(js_ast.SymbolOther, p.symbols[shadowRef.InnerIndex].OriginalName)
+			p.currentScope.Generated = append(p.currentScope.Generated, captureRef)
+			p.recordDeclaredSymbol(captureRef)
+			p.mergeSymbols(shadowRef, captureRef)
+			stmts = append(stmts, js_ast.Stmt{Loc: classLoc, Data: &js_ast.SLocal{
+				Kind: p.selectLocalKind(js_ast.LocalConst),
+				Decls: []js_ast.Decl{{
+					Binding: js_ast.Binding{Loc: name.Loc, Data: &js_ast.BIdentifier{Ref: captureRef}},
+					Value:   init,
+				}},
+			}})
+			init = &js_ast.Expr{Loc: classLoc, Data: &js_ast.EIdentifier{Ref: captureRef}}
+			p.recordUsage(captureRef)
+		} else {
+			// If there are class decorators, then we actually need to mutate the
+			// immutable "const" binding that shadows everything in the class body.
+			// The official TypeScript compiler does this by rewriting all class name
+			// references in the class body to another temporary variable. This is
+			// basically what we're doing here.
+			if shadowRef != js_ast.InvalidRef {
+				p.mergeSymbols(shadowRef, nameRef)
+			}
+		}
+
+		// Generate the variable statement that will represent the class statement
 		stmts = append(stmts, js_ast.Stmt{Loc: classLoc, Data: &js_ast.SLocal{
-			Kind:     js_ast.LocalLet,
+			Kind:     p.selectLocalKind(js_ast.LocalLet),
 			IsExport: kind == classKindExportStmt,
 			Decls: []js_ast.Decl{{
-				Binding: js_ast.Binding{Loc: name.Loc, Data: &js_ast.BIdentifier{Ref: id.Ref}},
-				Value:   &js_ast.Expr{Loc: classLoc, Data: &classExpr},
+				Binding: js_ast.Binding{Loc: name.Loc, Data: &js_ast.BIdentifier{Ref: nameRef}},
+				Value:   init,
 			}},
 		}})
 	} else {
@@ -1977,6 +2076,12 @@ func (p *parser) lowerClass(stmt js_ast.Stmt, expr js_ast.Expr) ([]js_ast.Stmt,
 				Value:       js_ast.ExprOrStmt{Stmt: &js_ast.Stmt{Loc: classLoc, Data: &js_ast.SClass{Class: *class}}},
 			}})
 		}
+
+		// The shadowing name inside the class statement should be the same as
+		// the class statement name itself
+		if class.Name != nil && shadowRef != js_ast.InvalidRef {
+			p.mergeSymbols(shadowRef, class.Name.Ref)
+		}
 	}
 
 	// The official TypeScript compiler adds generated code after the class body
@@ -1998,12 +2103,16 @@ func (p *parser) lowerClass(stmt js_ast.Stmt, expr js_ast.Expr) ([]js_ast.Stmt,
 	}
 	if len(class.TSDecorators) > 0 {
 		stmts = append(stmts, js_ast.AssignStmt(
-			nameFunc(),
+			js_ast.Expr{Loc: nameForClassDecorators.Loc, Data: &js_ast.EIdentifier{Ref: nameForClassDecorators.Ref}},
 			p.callRuntime(classLoc, "__decorate", []js_ast.Expr{
 				{Loc: classLoc, Data: &js_ast.EArray{Items: class.TSDecorators}},
-				nameFunc(),
+				{Loc: nameForClassDecorators.Loc, Data: &js_ast.EIdentifier{Ref: nameForClassDecorators.Ref}},
 			}),
 		))
+		p.recordUsage(nameForClassDecorators.Ref)
+		p.recordUsage(nameForClassDecorators.Ref)
+	}
+	if generatedLocalStmt {
 		if kind == classKindExportDefaultStmt {
 			// Generate a new default name symbol since the current one is being used
 			// by the class. If this SExportDefault turns into a variable declaration,
@@ -2018,13 +2127,21 @@ func (p *parser) lowerClass(stmt js_ast.Stmt, expr js_ast.Expr) ([]js_ast.Stmt,
 				Value:       js_ast.ExprOrStmt{Expr: &name},
 			}})
 		}
+
+		// Calling "nameFunc" will set the class name, but we don't want it to have
+		// one. If the class name was necessary, we would have already split it off
+		// into a "const" symbol above. Reset it back to empty here now that we
+		// know we won't call "nameFunc" after this point.
 		class.Name = nil
 	}
+	if keepNameStmt.Data != nil {
+		stmts = append(stmts, keepNameStmt)
+	}
 	return stmts, js_ast.Expr{}
 }
 
 func (p *parser) shouldLowerSuperPropertyAccess(expr js_ast.Expr) bool {
-	if p.fnOrArrowDataVisit.isAsync && p.UnsupportedJSFeatures.Has(compat.AsyncAwait) {
+	if p.fnOrArrowDataVisit.isAsync && p.options.unsupportedJSFeatures.Has(compat.AsyncAwait) {
 		_, isSuper := expr.Data.(*js_ast.ESuper)
 		return isSuper
 	}
