@@ -1,11 +1,12 @@
 import * as types from "./types";
 import * as common from "./common";
-import * as child_process from "child_process";
-import * as crypto from "crypto";
-import * as path from "path";
-import * as fs from "fs";
-import * as os from "os";
-import { isatty } from "tty";
+
+import child_process = require('child_process');
+import crypto = require('crypto');
+import path = require('path');
+import fs = require('fs');
+import os = require('os');
+import tty = require('tty');
 
 declare const ESBUILD_VERSION: string;
 
@@ -48,31 +49,54 @@ let esbuildCommandAndArgs = (): [string, string[]] => {
 };
 
 // Return true if stderr is a TTY
-let isTTY = () => isatty(2);
+let isTTY = () => tty.isatty(2);
 
 export let version = ESBUILD_VERSION;
 
 export let build: typeof types.build = options => {
   return startService().then(service => {
-    let promise = service.build(options);
-    promise.then(service.stop, service.stop); // Kill the service afterwards
-    return promise;
+    return service.build(options).then(result => {
+      if (result.rebuild) {
+        let old = result.rebuild.dispose;
+        result.rebuild.dispose = () => {
+          old();
+          service.stop();
+        };
+      }
+      else service.stop();
+      return result;
+    }, error => {
+      service.stop();
+      throw error;
+    });
+  });
+};
+
+export let serve: typeof types.serve = (serveOptions, buildOptions) => {
+  return startService().then(service => {
+    return service.serve(serveOptions, buildOptions).then(result => {
+      result.wait.then(service.stop, service.stop);
+      return result;
+    }, error => {
+      service.stop();
+      throw error;
+    });
   });
 };
 
 export let transform: typeof types.transform = (input, options) => {
   return startService().then(service => {
     let promise = service.transform(input, options);
-    promise.then(service.stop, service.stop); // Kill the service afterwards
+    promise.then(service.stop, service.stop);
     return promise;
   });
 };
 
 export let buildSync: typeof types.buildSync = options => {
   let result: types.BuildResult;
-  runServiceSync(service => service.build(options, isTTY(), (err, res) => {
+  runServiceSync(service => service.buildOrServe(null, options, isTTY(), (err, res) => {
     if (err) throw err;
-    result = res!;
+    result = res as types.BuildResult;
   }));
   return result!;
 };
@@ -133,8 +157,15 @@ export let startService: typeof types.startService = options => {
   return Promise.resolve({
     build: options =>
       new Promise((resolve, reject) =>
-        service.build(options, isTTY(), (err, res) =>
-          err ? reject(err) : resolve(res!))),
+        service.buildOrServe(null, options, isTTY(), (err, res) =>
+          err ? reject(err) : resolve(res as types.BuildResult))),
+    serve: (serveOptions, buildOptions) => {
+      if (serveOptions === null || typeof serveOptions !== 'object')
+        throw new Error('The first argument must be an object')
+      return new Promise((resolve, reject) =>
+        service.buildOrServe(serveOptions, buildOptions, isTTY(), (err, res) =>
+          err ? reject(err) : resolve(res as types.ServeResult)))
+    },
     transform: (input, options) =>
       new Promise((resolve, reject) =>
         service.transform(input, options || {}, isTTY(), {
