@@ -13,7 +13,23 @@ export interface BuildRequest {
   write: boolean;
   stdin: string | null;
   resolveDir: string | null;
+  incremental: boolean;
   plugins?: BuildPlugin[];
+  serve?: ServeRequest;
+}
+
+export interface ServeRequest {
+  serveID: number;
+  port?: number;
+}
+
+export interface ServeResponse {
+  port: number;
+}
+
+export interface ServeStopRequest {
+  command: 'serve-stop';
+  serveID: number;
 }
 
 export interface BuildPlugin {
@@ -26,6 +42,7 @@ export interface BuildResponse {
   errors: types.Message[];
   warnings: types.Message[];
   outputFiles: BuildOutputFile[];
+  rebuildID?: number;
 }
 
 export interface BuildOutputFile {
@@ -33,6 +50,28 @@ export interface BuildOutputFile {
   contents: Uint8Array;
 }
 
+export interface RebuildRequest {
+  command: 'rebuild';
+  rebuildID: number;
+}
+
+export interface RebuildDisposeRequest {
+  command: 'rebuild-dispose';
+  rebuildID: number;
+}
+
+export interface OnRequestRequest {
+  command: 'serve-request';
+  serveID: number;
+  args: types.ServeOnRequestArgs;
+}
+
+export interface OnWaitRequest {
+  command: 'serve-wait';
+  serveID: number;
+  error: string | null;
+}
+
 export interface TransformRequest {
   command: 'transform';
   flags: string[];
