@@ -3,6 +3,7 @@ export type Format = 'iife' | 'cjs' | 'esm';
 export type Loader = 'js' | 'jsx' | 'ts' | 'tsx' | 'css' | 'json' | 'text' | 'base64' | 'file' | 'dataurl' | 'binary' | 'default';
 export type LogLevel = 'info' | 'warning' | 'error' | 'silent';
 export type Charset = 'ascii' | 'utf8';
+export type TreeShaking = true | 'ignore-annotations';
 
 interface CommonOptions {
   sourcemap?: boolean | 'inline' | 'external';
@@ -15,6 +16,7 @@ interface CommonOptions {
   minifyIdentifiers?: boolean;
   minifySyntax?: boolean;
   charset?: Charset;
+  treeShaking?: TreeShaking;
 
   jsxFactory?: string;
   jsxFragment?: string;
@@ -22,6 +24,8 @@ interface CommonOptions {
   pure?: string[];
   avoidTDZ?: boolean;
   keepNames?: boolean;
+  banner?: string;
+  footer?: string;
 
   color?: boolean;
   logLevel?: LogLevel;
@@ -45,7 +49,7 @@ export interface BuildOptions extends CommonOptions {
   outExtension?: { [ext: string]: string };
   publicPath?: string;
   inject?: string[];
-
+  incremental?: boolean;
   entryPoints?: string[];
   stdin?: StdinOptions;
   plugins?: Plugin[];
@@ -81,6 +85,7 @@ export interface OutputFile {
 export interface BuildResult {
   warnings: Message[];
   outputFiles?: OutputFile[]; // Only when "write: false"
+  rebuild?: (() => Promise<BuildResult>) & { dispose(): void }; // Only when "incremental" is true
 }
 
 export interface BuildFailure extends Error {
@@ -88,6 +93,25 @@ export interface BuildFailure extends Error {
   warnings: Message[];
 }
 
+export interface ServeOptions {
+  port?: number;
+  onRequest?: (args: ServeOnRequestArgs) => void;
+}
+
+export interface ServeOnRequestArgs {
+  remoteAddress: string;
+  method: string;
+  path: string;
+  status: number;
+  timeInMS: number; // The time to generate the response, not to send it
+}
+
+export interface ServeResult {
+  port: number;
+  wait: Promise<void>;
+  stop: () => void;
+}
+
 export interface TransformOptions extends CommonOptions {
   tsconfigRaw?: string | {
     compilerOptions?: {
@@ -195,12 +219,14 @@ export interface Metadata {
       imports: {
         path: string
       }[]
+      exports: string[]
     }
   }
 }
 
 export interface Service {
   build(options: BuildOptions): Promise<BuildResult>;
+  serve(serveOptions: ServeOptions, buildOptions: BuildOptions): Promise<ServeResult>;
   transform(input: string, options?: TransformOptions): Promise<TransformResult>;
 
   // This stops the service, which kills the long-lived child process. Any
@@ -216,6 +242,13 @@ export interface Service {
 // Works in browser: no
 export declare function build(options: BuildOptions): Promise<BuildResult>;
 
+// This function is similar to "build" but it serves the resulting files over
+// HTTP on a localhost address with the specified port.
+//
+// Works in node: yes
+// Works in browser: no
+export declare function serve(serveOptions: ServeOptions, buildOptions: BuildOptions): Promise<ServeResult>;
+
 // This function transforms a single JavaScript file. It can be used to minify
 // JavaScript, convert TypeScript/JSX to JavaScript, or convert newer JavaScript
 // to older JavaScript. It returns a promise that is either resolved with a
