@@ -3,15 +3,22 @@ package api
 import (
 	"fmt"
 	"io/ioutil"
+	"math/rand"
+	"mime"
+	"net"
+	"net/http"
 	"os"
+	"path"
 	"path/filepath"
 	"regexp"
 	"sort"
 	"strconv"
 	"strings"
 	"sync"
+	"time"
 
 	"github.com/evanw/esbuild/internal/bundler"
+	"github.com/evanw/esbuild/internal/cache"
 	"github.com/evanw/esbuild/internal/compat"
 	"github.com/evanw/esbuild/internal/config"
 	"github.com/evanw/esbuild/internal/fs"
@@ -102,6 +109,17 @@ func validateASCIIOnly(value Charset) bool {
 	}
 }
 
+func validateIgnoreDCEAnnotations(value TreeShaking) bool {
+	switch value {
+	case TreeShakingDefault:
+		return false
+	case TreeShakingIgnoreAnnotations:
+		return true
+	default:
+		panic("Invalid tree shaking")
+	}
+}
+
 func validateLoader(value Loader) config.Loader {
 	switch value {
 	case LoaderNone:
@@ -237,7 +255,16 @@ func validateExternals(log logger.Log, fs fs.FS, paths []string) config.External
 		AbsPaths:    make(map[string]bool),
 	}
 	for _, path := range paths {
-		if resolver.IsPackagePath(path) {
+		if index := strings.IndexByte(path, '*'); index != -1 {
+			if strings.ContainsRune(path[index+1:], '*') {
+				log.AddError(nil, logger.Loc{}, fmt.Sprintf("External path %q cannot have more than one \"*\" wildcard", path))
+			} else {
+				result.Patterns = append(result.Patterns, config.WildcardPattern{
+					Prefix: path[:index],
+					Suffix: path[index+1:],
+				})
+			}
+		} else if resolver.IsPackagePath(path) {
 			result.NodeModules[path] = true
 		} else if absPath := validatePath(log, fs, path); absPath != "" {
 			result.AbsPaths[absPath] = true
@@ -320,7 +347,7 @@ func validateDefines(log logger.Log, defines map[string]string, pureFns []string
 
 		// Parse the value as JSON
 		source := logger.Source{Contents: value}
-		expr, ok := js_parser.ParseJSON(logger.NewDeferLog(), source, js_parser.ParseJSONOptions{})
+		expr, ok := js_parser.ParseJSON(logger.NewDeferLog(), source, js_parser.JSONOptions{})
 		if !ok {
 			log.AddError(nil, logger.Loc{}, fmt.Sprintf("Invalid define value: %q", value))
 			continue
@@ -377,18 +404,21 @@ func validatePath(log logger.Log, fs fs.FS, relPath string) string {
 	return absPath
 }
 
-func validateOutputExtensions(log logger.Log, outExtensions map[string]string) map[string]string {
-	result := make(map[string]string)
+func validateOutputExtensions(log logger.Log, outExtensions map[string]string) (js string, css string) {
 	for key, value := range outExtensions {
-		if key != ".js" && key != ".css" {
-			log.AddError(nil, logger.Loc{}, fmt.Sprintf("Invalid output extension: %q (valid: .css, .js)", key))
-		}
 		if !isValidExtension(value) {
 			log.AddError(nil, logger.Loc{}, fmt.Sprintf("Invalid output extension: %q", value))
 		}
-		result[key] = value
+		switch key {
+		case ".js":
+			js = value
+		case ".css":
+			css = value
+		default:
+			log.AddError(nil, logger.Loc{}, fmt.Sprintf("Invalid output extension: %q (valid: .css, .js)", key))
+		}
 	}
-	return result
+	return
 }
 
 func convertMessagesToPublic(kind logger.MsgKind, msgs []logger.Msg) []Message {
@@ -396,7 +426,7 @@ func convertMessagesToPublic(kind logger.MsgKind, msgs []logger.Msg) []Message {
 	for _, msg := range msgs {
 		if msg.Kind == kind {
 			var location *Location
-			loc := msg.Location
+			loc := msg.Data.Location
 
 			if loc != nil {
 				location = &Location{
@@ -410,7 +440,7 @@ func convertMessagesToPublic(kind logger.MsgKind, msgs []logger.Msg) []Message {
 			}
 
 			filtered = append(filtered, Message{
-				Text:     msg.Text,
+				Text:     msg.Data.Text,
 				Location: location,
 			})
 		}
@@ -439,9 +469,11 @@ func convertMessagesToInternal(msgs []logger.Msg, kind logger.MsgKind, messages
 		}
 
 		msgs = append(msgs, logger.Msg{
-			Kind:     kind,
-			Text:     message.Text,
-			Location: location,
+			Kind: kind,
+			Data: logger.MsgData{
+				Text:     message.Text,
+				Location: location,
+			},
 		})
 	}
 	return msgs
@@ -451,16 +483,31 @@ func convertMessagesToInternal(msgs []logger.Msg, kind logger.MsgKind, messages
 // Build API
 
 func buildImpl(buildOpts BuildOptions) BuildResult {
-	log := logger.NewStderrLog(logger.StderrOptions{
+	logOptions := logger.StderrOptions{
 		IncludeSource: true,
 		ErrorLimit:    buildOpts.ErrorLimit,
 		Color:         validateColor(buildOpts.Color),
 		LogLevel:      validateLogLevel(buildOpts.LogLevel),
-	})
+	}
+	log := logger.NewStderrLog(logOptions)
+	realFS := fs.RealFS()
 
+	// Do not re-evaluate plugins when rebuilding
+	plugins := loadPlugins(realFS, log, buildOpts.Plugins)
+	return rebuildImpl(buildOpts, cache.MakeCacheSet(), plugins, logOptions, log)
+}
+
+func rebuildImpl(
+	buildOpts BuildOptions,
+	caches *cache.CacheSet,
+	plugins []config.Plugin,
+	logOptions logger.StderrOptions,
+	log logger.Log,
+) BuildResult {
 	// Convert and validate the buildOpts
 	realFS := fs.RealFS()
 	jsFeatures, cssFeatures := validateFeatures(log, buildOpts.Target, buildOpts.Engines)
+	outJS, outCSS := validateOutputExtensions(log, buildOpts.OutExtensions)
 	options := config.Options{
 		UnsupportedJSFeatures:  jsFeatures,
 		UnsupportedCSSFeatures: cssFeatures,
@@ -468,30 +515,34 @@ func buildImpl(buildOpts BuildOptions) BuildResult {
 			Factory:  validateJSX(log, buildOpts.JSXFactory, "factory"),
 			Fragment: validateJSX(log, buildOpts.JSXFragment, "fragment"),
 		},
-		Defines:           validateDefines(log, buildOpts.Define, buildOpts.Pure),
-		Platform:          validatePlatform(buildOpts.Platform),
-		SourceMap:         validateSourceMap(buildOpts.Sourcemap),
-		MangleSyntax:      buildOpts.MinifySyntax,
-		RemoveWhitespace:  buildOpts.MinifyWhitespace,
-		MinifyIdentifiers: buildOpts.MinifyIdentifiers,
-		ASCIIOnly:         validateASCIIOnly(buildOpts.Charset),
-		ModuleName:        validateGlobalName(log, buildOpts.GlobalName),
-		CodeSplitting:     buildOpts.Splitting,
-		OutputFormat:      validateFormat(buildOpts.Format),
-		AbsOutputFile:     validatePath(log, realFS, buildOpts.Outfile),
-		AbsOutputDir:      validatePath(log, realFS, buildOpts.Outdir),
-		AbsOutputBase:     validatePath(log, realFS, buildOpts.Outbase),
-		AbsMetadataFile:   validatePath(log, realFS, buildOpts.Metafile),
-		OutputExtensions:  validateOutputExtensions(log, buildOpts.OutExtensions),
-		ExtensionToLoader: validateLoaders(log, buildOpts.Loader),
-		ExtensionOrder:    validateResolveExtensions(log, buildOpts.ResolveExtensions),
-		ExternalModules:   validateExternals(log, realFS, buildOpts.External),
-		TsConfigOverride:  validatePath(log, realFS, buildOpts.Tsconfig),
-		MainFields:        buildOpts.MainFields,
-		PublicPath:        buildOpts.PublicPath,
-		AvoidTDZ:          buildOpts.AvoidTDZ,
-		KeepNames:         buildOpts.KeepNames,
-		InjectAbsPaths:    make([]string, len(buildOpts.Inject)),
+		Defines:              validateDefines(log, buildOpts.Define, buildOpts.Pure),
+		Platform:             validatePlatform(buildOpts.Platform),
+		SourceMap:            validateSourceMap(buildOpts.Sourcemap),
+		MangleSyntax:         buildOpts.MinifySyntax,
+		RemoveWhitespace:     buildOpts.MinifyWhitespace,
+		MinifyIdentifiers:    buildOpts.MinifyIdentifiers,
+		ASCIIOnly:            validateASCIIOnly(buildOpts.Charset),
+		IgnoreDCEAnnotations: validateIgnoreDCEAnnotations(buildOpts.TreeShaking),
+		GlobalName:           validateGlobalName(log, buildOpts.GlobalName),
+		CodeSplitting:        buildOpts.Splitting,
+		OutputFormat:         validateFormat(buildOpts.Format),
+		AbsOutputFile:        validatePath(log, realFS, buildOpts.Outfile),
+		AbsOutputDir:         validatePath(log, realFS, buildOpts.Outdir),
+		AbsOutputBase:        validatePath(log, realFS, buildOpts.Outbase),
+		AbsMetadataFile:      validatePath(log, realFS, buildOpts.Metafile),
+		OutputExtensionJS:    outJS,
+		OutputExtensionCSS:   outCSS,
+		ExtensionToLoader:    validateLoaders(log, buildOpts.Loader),
+		ExtensionOrder:       validateResolveExtensions(log, buildOpts.ResolveExtensions),
+		ExternalModules:      validateExternals(log, realFS, buildOpts.External),
+		TsConfigOverride:     validatePath(log, realFS, buildOpts.Tsconfig),
+		MainFields:           buildOpts.MainFields,
+		PublicPath:           buildOpts.PublicPath,
+		KeepNames:            buildOpts.KeepNames,
+		InjectAbsPaths:       make([]string, len(buildOpts.Inject)),
+		Banner:               buildOpts.Banner,
+		Footer:               buildOpts.Footer,
+		Plugins:              plugins,
 	}
 	for i, path := range buildOpts.Inject {
 		options.InjectAbsPaths[i] = validatePath(log, realFS, path)
@@ -574,15 +625,13 @@ func buildImpl(buildOpts BuildOptions) BuildResult {
 		log.AddError(nil, logger.Loc{}, "Splitting currently only works with the \"esm\" format")
 	}
 
-	loadPlugins(&options, realFS, log, buildOpts.Plugins)
-
 	var outputFiles []OutputFile
 
 	// Stop now if there were errors
 	if !log.HasErrors() {
 		// Scan over the bundle
-		resolver := resolver.NewResolver(realFS, log, options)
-		bundle := bundler.ScanBundle(log, realFS, resolver, entryPaths, options)
+		resolver := resolver.NewResolver(realFS, log, caches, options)
+		bundle := bundler.ScanBundle(log, realFS, resolver, caches, entryPaths, options)
 
 		// Stop now if there were errors
 		if !log.HasErrors() {
@@ -644,11 +693,19 @@ func buildImpl(buildOpts BuildOptions) BuildResult {
 		}
 	}
 
+	var rebuild func() BuildResult
+	if buildOpts.Incremental {
+		rebuild = func() BuildResult {
+			return rebuildImpl(buildOpts, caches, plugins, logOptions, logger.NewStderrLog(logOptions))
+		}
+	}
+
 	msgs := log.Done()
 	return BuildResult{
 		Errors:      convertMessagesToPublic(logger.Error, msgs),
 		Warnings:    convertMessagesToPublic(logger.Warning, msgs),
 		OutputFiles: outputFiles,
+		Rebuild:     rebuild,
 	}
 }
 
@@ -672,13 +729,14 @@ func transformImpl(input string, transformOpts TransformOptions) TransformResult
 	}
 
 	// Settings from "tsconfig.json" override those
+	caches := cache.MakeCacheSet()
 	if transformOpts.TsconfigRaw != "" {
 		source := logger.Source{
 			KeyPath:    logger.Path{Text: "tsconfig.json"},
 			PrettyPath: "tsconfig.json",
 			Contents:   transformOpts.TsconfigRaw,
 		}
-		if result := resolver.ParseTSConfigJSON(log, source, nil); result != nil {
+		if result := resolver.ParseTSConfigJSON(log, source, &caches.JSONCache, nil); result != nil {
 			if len(result.JSXFactory) > 0 {
 				jsx.Factory = result.JSXFactory
 			}
@@ -711,13 +769,13 @@ func transformImpl(input string, transformOpts TransformOptions) TransformResult
 		Defines:                 validateDefines(log, transformOpts.Define, transformOpts.Pure),
 		SourceMap:               validateSourceMap(transformOpts.Sourcemap),
 		OutputFormat:            validateFormat(transformOpts.Format),
-		ModuleName:              validateGlobalName(log, transformOpts.GlobalName),
+		GlobalName:              validateGlobalName(log, transformOpts.GlobalName),
 		MangleSyntax:            transformOpts.MinifySyntax,
 		RemoveWhitespace:        transformOpts.MinifyWhitespace,
 		MinifyIdentifiers:       transformOpts.MinifyIdentifiers,
 		ASCIIOnly:               validateASCIIOnly(transformOpts.Charset),
+		IgnoreDCEAnnotations:    validateIgnoreDCEAnnotations(transformOpts.TreeShaking),
 		AbsOutputFile:           transformOpts.Sourcefile + "-out",
-		AvoidTDZ:                transformOpts.AvoidTDZ,
 		KeepNames:               transformOpts.KeepNames,
 		UseDefineForClassFields: useDefineForClassFieldsTS,
 		PreserveUnusedImportsTS: preserveUnusedImportsTS,
@@ -726,6 +784,8 @@ func transformImpl(input string, transformOpts TransformOptions) TransformResult
 			Contents:   input,
 			SourceFile: transformOpts.Sourcefile,
 		},
+		Banner: transformOpts.Banner,
+		Footer: transformOpts.Footer,
 	}
 	if options.SourceMap == config.SourceMapLinkedWithComment {
 		// Linked source maps don't make sense because there's no output file name
@@ -747,8 +807,8 @@ func transformImpl(input string, transformOpts TransformOptions) TransformResult
 	if !log.HasErrors() {
 		// Scan over the bundle
 		mockFS := fs.MockFS(make(map[string]string))
-		resolver := resolver.NewResolver(mockFS, log, options)
-		bundle := bundler.ScanBundle(log, mockFS, resolver, nil, options)
+		resolver := resolver.NewResolver(mockFS, log, caches, options)
+		bundle := bundler.ScanBundle(log, mockFS, resolver, caches, nil, options)
 
 		// Stop now if there were errors
 		if !log.HasErrors() {
@@ -821,10 +881,10 @@ func (impl *pluginImpl) OnResolve(options OnResolveOptions, callback func(OnReso
 
 			// Convert log messages
 			if len(response.Errors)+len(response.Warnings) > 0 {
-				msgs := make(sortableMsgs, 0, len(response.Errors)+len(response.Warnings))
+				msgs := make(logger.SortableMsgs, 0, len(response.Errors)+len(response.Warnings))
 				msgs = convertMessagesToInternal(msgs, logger.Error, response.Errors)
 				msgs = convertMessagesToInternal(msgs, logger.Warning, response.Warnings)
-				sort.Sort(msgs)
+				sort.Stable(msgs)
 				result.Msgs = msgs
 			}
 			return
@@ -862,10 +922,10 @@ func (impl *pluginImpl) OnLoad(options OnLoadOptions, callback func(OnLoadArgs)
 
 			// Convert log messages
 			if len(response.Errors)+len(response.Warnings) > 0 {
-				msgs := make(sortableMsgs, 0, len(response.Errors)+len(response.Warnings))
+				msgs := make(logger.SortableMsgs, 0, len(response.Errors)+len(response.Warnings))
 				msgs = convertMessagesToInternal(msgs, logger.Error, response.Errors)
 				msgs = convertMessagesToInternal(msgs, logger.Warning, response.Warnings)
-				sort.Sort(msgs)
+				sort.Stable(msgs)
 				result.Msgs = msgs
 			}
 			return
@@ -873,31 +933,7 @@ func (impl *pluginImpl) OnLoad(options OnLoadOptions, callback func(OnLoadArgs)
 	})
 }
 
-// This type is just so we can use Go's native sort function
-type sortableMsgs []logger.Msg
-
-func (a sortableMsgs) Len() int          { return len(a) }
-func (a sortableMsgs) Swap(i int, j int) { a[i], a[j] = a[j], a[i] }
-
-func (a sortableMsgs) Less(i int, j int) bool {
-	ai := a[i].Location
-	aj := a[j].Location
-	if ai == nil || aj == nil {
-		return ai == nil && aj != nil
-	}
-	if ai.File != aj.File {
-		return ai.File < aj.File
-	}
-	if ai.Line != aj.Line {
-		return ai.Line < aj.Line
-	}
-	if ai.Column != aj.Column {
-		return ai.Column < aj.Column
-	}
-	return a[i].Text < a[j].Text
-}
-
-func loadPlugins(options *config.Options, fs fs.FS, log logger.Log, plugins []Plugin) {
+func loadPlugins(fs fs.FS, log logger.Log, plugins []Plugin) (results []config.Plugin) {
 	for i, item := range plugins {
 		if item.Name == "" {
 			log.AddError(nil, logger.Loc{}, fmt.Sprintf("Plugin at index %d is missing a name", i))
@@ -911,6 +947,258 @@ func loadPlugins(options *config.Options, fs fs.FS, log logger.Log, plugins []Pl
 		}
 
 		item.Setup(impl)
-		options.Plugins = append(options.Plugins, impl.plugin)
+		results = append(results, impl.plugin)
+	}
+	return
+}
+
+////////////////////////////////////////////////////////////////////////////////
+// Serve API
+
+type apiHandler struct {
+	mutex        sync.Mutex
+	outdir       string
+	onRequest    func(ServeOnRequestArgs)
+	rebuild      func() BuildResult
+	currentBuild *runningBuild
+}
+
+type runningBuild struct {
+	waitGroup sync.WaitGroup
+	result    BuildResult
+}
+
+func (h *apiHandler) build() BuildResult {
+	build := func() *runningBuild {
+		h.mutex.Lock()
+		defer h.mutex.Unlock()
+		if h.currentBuild == nil {
+			build := &runningBuild{}
+			build.waitGroup.Add(1)
+			h.currentBuild = build
+
+			// Build on another thread
+			go func() {
+				result := h.rebuild()
+				h.rebuild = result.Rebuild
+				build.result = result
+				build.waitGroup.Done()
+
+				// Build results stay valid for a little bit afterward since a page
+				// load may involve multiple requests and don't want to rebuild
+				// separately for each of those requests.
+				time.Sleep(250 * time.Millisecond)
+				h.mutex.Lock()
+				defer h.mutex.Unlock()
+				h.currentBuild = nil
+			}()
+		}
+		return h.currentBuild
+	}()
+	build.waitGroup.Wait()
+	return build.result
+}
+
+func escapeForHTML(text string) string {
+	text = strings.ReplaceAll(text, "&", "&amp;")
+	text = strings.ReplaceAll(text, "<", "&lt;")
+	text = strings.ReplaceAll(text, ">", "&gt;")
+	return text
+}
+
+func escapeForAttribute(text string) string {
+	text = escapeForHTML(text)
+	text = strings.ReplaceAll(text, "\"", "&quot;")
+	text = strings.ReplaceAll(text, "'", "&apos;")
+	return text
+}
+
+func (h *apiHandler) notifyRequest(duration time.Duration, req *http.Request, status int) {
+	if h.onRequest != nil {
+		h.onRequest(ServeOnRequestArgs{
+			RemoteAddress: req.RemoteAddr,
+			Method:        req.Method,
+			Path:          req.URL.Path,
+			Status:        status,
+			TimeInMS:      int(duration.Milliseconds()),
+		})
 	}
 }
+
+func errorsToString(errors []Message) string {
+	stderrOptions := logger.StderrOptions{IncludeSource: true}
+	terminalOptions := logger.TerminalInfo{}
+	sb := strings.Builder{}
+	limit := 5
+	for i, msg := range convertMessagesToInternal(nil, logger.Error, errors) {
+		if i == limit {
+			sb.WriteString(fmt.Sprintf("%d out of %d errors shown\n", limit, len(errors)))
+			break
+		}
+		sb.WriteString(msg.String(stderrOptions, terminalOptions))
+	}
+	return sb.String()
+}
+
+func (h *apiHandler) ServeHTTP(res http.ResponseWriter, req *http.Request) {
+	start := time.Now()
+
+	// Handle get requests
+	if req.Method == "GET" && strings.HasPrefix(req.URL.Path, "/") {
+		res.Header().Set("Access-Control-Allow-Origin", "*")
+		queryIsDir := false
+		fileEntries := []string{}
+		dirEntries := []string{}
+		isDirEntry := make(map[string]bool)
+		queryPath := req.URL.Path[1:]
+		if strings.HasSuffix(queryPath, "/") {
+			queryPath = queryPath[:len(queryPath)-1]
+		}
+		queryDir := queryPath
+		if queryDir != "" {
+			queryDir += "/"
+		}
+		queryExt := path.Ext(queryPath)
+		result := h.build()
+
+		// Requests fail if the build had errors
+		if len(result.Errors) > 0 {
+			go h.notifyRequest(time.Since(start), req, http.StatusServiceUnavailable)
+			res.Header().Set("Content-Type", "text/plain; charset=utf-8")
+			res.WriteHeader(http.StatusServiceUnavailable)
+			res.Write([]byte(errorsToString(result.Errors)))
+			return
+		}
+
+		// Check the output files for a match
+		for _, file := range result.OutputFiles {
+			if relPath, err := filepath.Rel(h.outdir, file.Path); err == nil {
+				relPath = strings.ReplaceAll(relPath, "\\", "/")
+
+				// An exact match
+				if relPath == queryPath {
+					if contentType := mime.TypeByExtension(queryExt); contentType != "" {
+						res.Header().Set("Content-Type", contentType)
+					}
+					go h.notifyRequest(time.Since(start), req, http.StatusOK)
+					res.Write(file.Contents)
+					return
+				}
+
+				// A match inside this directory
+				if strings.HasPrefix(relPath, queryDir) {
+					entry := relPath[len(queryDir):]
+					queryIsDir = true
+					if slash := strings.IndexByte(entry, '/'); slash == -1 {
+						fileEntries = append(fileEntries, entry)
+					} else if dir := entry[:slash]; !isDirEntry[dir] {
+						dirEntries = append(dirEntries, dir)
+						isDirEntry[dir] = true
+					}
+				}
+			}
+		}
+
+		// Directory listing
+		if queryIsDir {
+			html := strings.Builder{}
+			html.WriteString(`<!DOCTYPE html>`)
+			html.WriteString(`<title>`)
+			html.WriteString(escapeForHTML(queryDir))
+			html.WriteString(`</title>`)
+			html.WriteString(`<ul>`)
+			if queryPath != "" {
+				html.WriteString(fmt.Sprintf(`<li><a href="%s">../</a></li>`, escapeForAttribute(path.Dir("/"+queryPath))))
+			}
+			sort.Strings(dirEntries)
+			for _, entry := range dirEntries {
+				html.WriteString(fmt.Sprintf(`<li><a href="/%s">%s/</a></li>`, escapeForAttribute(path.Join(queryPath, entry)), escapeForHTML(entry)))
+			}
+			sort.Strings(fileEntries)
+			for _, entry := range fileEntries {
+				html.WriteString(fmt.Sprintf(`<li><a href="/%s">%s</a></li>`, escapeForAttribute(path.Join(queryPath, entry)), escapeForHTML(entry)))
+			}
+			html.WriteString(`</ul>`)
+			res.Header().Set("Content-Type", "text/html; charset=utf-8")
+			go h.notifyRequest(time.Since(start), req, http.StatusOK)
+			res.Write([]byte(html.String()))
+			return
+		}
+	}
+
+	// Default to a 404
+	res.Header().Set("Content-Type", "text/plain; charset=utf-8")
+	go h.notifyRequest(time.Since(start), req, http.StatusNotFound)
+	res.WriteHeader(http.StatusNotFound)
+	res.Write([]byte("404 - Not Found"))
+}
+
+func serveImpl(serveOptions ServeOptions, buildOptions BuildOptions) (ServeResult, error) {
+	// The output directory isn't actually ever written to. It just needs to be
+	// very unlikely to be used as a source file so it doesn't collide.
+	outdir := filepath.Join(os.TempDir(), strconv.FormatInt(rand.NewSource(time.Now().Unix()).Int63(), 36))
+	buildOptions.Incremental = true
+	buildOptions.Write = false
+	buildOptions.Outfile = ""
+	buildOptions.Outdir = outdir
+
+	// Pick the port
+	var listener net.Listener
+	host := "127.0.0.1"
+	if serveOptions.Port == 0 {
+		// Default to picking a "800X" port
+		for port := 8000; port <= 8009; port++ {
+			if result, err := net.Listen("tcp", fmt.Sprintf("%s:%d", host, port)); err == nil {
+				listener = result
+				break
+			}
+		}
+	}
+	if listener == nil {
+		// Otherwise pick the provided port
+		if result, err := net.Listen("tcp", fmt.Sprintf("%s:%d", host, serveOptions.Port)); err != nil {
+			return ServeResult{}, err
+		} else {
+			listener = result
+		}
+	}
+
+	// Try listening on the provided port
+	addr := listener.Addr().String()
+
+	// Extract the real port in case we passed a port of "0"
+	var result ServeResult
+	if _, text, err := net.SplitHostPort(addr); err == nil {
+		if port, err := strconv.ParseInt(text, 10, 32); err == nil {
+			result.Port = uint16(port)
+		}
+	}
+
+	// The first build will just build normally
+	handler := &apiHandler{
+		outdir:    outdir,
+		onRequest: serveOptions.OnRequest,
+		rebuild:   func() BuildResult { return Build(buildOptions) },
+	}
+
+	// Start the server
+	server := &http.Server{Addr: addr, Handler: handler}
+	wait := make(chan error, 1)
+	result.Wait = func() error { return <-wait }
+	result.Stop = func() { server.Close() }
+	go func() {
+		if err := server.Serve(listener); err != http.ErrServerClosed {
+			wait <- err
+		} else {
+			wait <- nil
+		}
+	}()
+
+	// Start the first build shortly after this function returns (but not
+	// immediately so that stuff we print right after this will come first)
+	go func() {
+		time.Sleep(10 * time.Millisecond)
+		handler.build()
+	}()
+	return result, nil
+}
