@@ -87,6 +87,21 @@ func parseOptionsImpl(osArgs []string, buildOpts *api.BuildOptions, transformOpt
 				return fmt.Errorf("Invalid charset value: %q (valid: ascii, utf8)", name)
 			}
 
+		case strings.HasPrefix(arg, "--tree-shaking="):
+			var value *api.TreeShaking
+			if buildOpts != nil {
+				value = &buildOpts.TreeShaking
+			} else {
+				value = &transformOpts.TreeShaking
+			}
+			name := arg[len("--tree-shaking="):]
+			switch name {
+			case "ignore-annotations":
+				*value = api.TreeShakingIgnoreAnnotations
+			default:
+				return fmt.Errorf("Invalid tree shaking value: %q (valid: ignore-annotations)", name)
+			}
+
 		case arg == "--avoid-tdz":
 			if buildOpts != nil {
 				buildOpts.AvoidTDZ = true
@@ -302,6 +317,22 @@ func parseOptionsImpl(osArgs []string, buildOpts *api.BuildOptions, transformOpt
 				transformOpts.JSXFragment = value
 			}
 
+		case strings.HasPrefix(arg, "--banner="):
+			value := arg[len("--banner="):]
+			if buildOpts != nil {
+				buildOpts.Banner = value
+			} else {
+				transformOpts.Banner = value
+			}
+
+		case strings.HasPrefix(arg, "--footer="):
+			value := arg[len("--footer="):]
+			if buildOpts != nil {
+				buildOpts.Footer = value
+			} else {
+				transformOpts.Footer = value
+			}
+
 		case strings.HasPrefix(arg, "--error-limit="):
 			value := arg[len("--error-limit="):]
 			limit, err := strconv.Atoi(value)
@@ -354,6 +385,9 @@ func parseOptionsImpl(osArgs []string, buildOpts *api.BuildOptions, transformOpt
 				transformOpts.LogLevel = logLevel
 			}
 
+		case strings.HasPrefix(arg, "'--"):
+			return fmt.Errorf("Unexpected single quote character before flag (use \\\" to escape double quotes): %s", arg)
+
 		case !strings.HasPrefix(arg, "-") && buildOpts != nil:
 			buildOpts.EntryPoints = append(buildOpts.EntryPoints, arg)
 
@@ -465,6 +499,22 @@ func parseOptionsForRun(osArgs []string) (*api.BuildOptions, *api.TransformOptio
 }
 
 func runImpl(osArgs []string) int {
+	// Special-case running a server
+	for i, arg := range osArgs {
+		if arg == "--serve" {
+			arg = "--serve=0"
+		}
+		if strings.HasPrefix(arg, "--serve=") {
+			serve := arg[len("--serve="):]
+			osArgs = append(append([]string{}, osArgs[:i]...), osArgs[i+1:]...)
+			if err := serveImpl(serve, osArgs); err != nil {
+				logger.PrintErrorToStderr(osArgs, err.Error())
+				return 1
+			}
+			return 0
+		}
+	}
+
 	buildOptions, transformOptions, err := parseOptionsForRun(osArgs)
 
 	switch {
@@ -524,3 +574,51 @@ func runImpl(osArgs []string) int {
 
 	return 0
 }
+
+func serveImpl(portText string, osArgs []string) error {
+	port, err := strconv.ParseInt(portText, 10, 32)
+	if err != nil {
+		return err
+	}
+	if port < 0 || port > 0xFFFF {
+		return fmt.Errorf("Invalid port number: %s", portText)
+	}
+
+	options := newBuildOptions()
+
+	// Apply defaults appropriate for the CLI
+	options.ErrorLimit = 5
+	options.LogLevel = api.LogLevelInfo
+
+	if err := parseOptionsImpl(osArgs, &options, nil); err != nil {
+		logger.PrintErrorToStderr(osArgs, err.Error())
+		return err
+	}
+
+	serveOptions := api.ServeOptions{
+		Port: uint16(port),
+		OnRequest: func(args api.ServeOnRequestArgs) {
+			logger.PrintTextToStderr(logger.LevelInfo, osArgs, func(colors logger.Colors) string {
+				statusColor := colors.Red
+				if args.Status == 200 {
+					statusColor = colors.Green
+				}
+				return fmt.Sprintf("%s%s - %q %s%d%s [%dms]%s\n",
+					colors.Dim, args.RemoteAddress, args.Method+" "+args.Path,
+					statusColor, args.Status, colors.Dim, args.TimeInMS, colors.Default)
+			})
+		},
+	}
+
+	result, err := api.Serve(serveOptions, options)
+	if err != nil {
+		return err
+	}
+
+	// Show what actually got bound if the port was 0
+	logger.PrintTextToStderr(logger.LevelInfo, osArgs, func(colors logger.Colors) string {
+		return fmt.Sprintf("%s\n > %shttp://localhost:%d/%s\n\n",
+			colors.Default, colors.Underline, result.Port, colors.Default)
+	})
+	return result.Wait()
+}
